// SCRIPT NAME: vBumpShadow
var VERSION=1.6;
// AUTHOR: James A. Taylor
// COPYRIGHT: James A. Taylor
// DESCRIPTION: Creates an Fake Shadow (Ambient Occlusion style) Map based on a greyscale height map.

#target photoshop
app.bringToFront();

//GLOBAL VARIABLES
var thepsd,thethumb,mybackground,theduplicate;
// preferences variables
var prefpath,preffile,prefs,previewimage;
var previewres, presets, presetlist;
// Photoshop state storage vars
var obcol,ofcol,myunit;

//START THE SCRIPT
main();

//FUNCTIONS

/*
------------
FUNCTION: main
------------
*/
function main()
	{
	// check if there are actually any documents open on which to perform the action
	if (documents.length <= 0)
		{
		alert("Sorry, This script requires a currently selected document.");
		return;
		}
	//set thepsd to refer to the current active document
	thepsd = activeDocument;
	//store the default unit type and set the current preference to be pixels
	myunit = preferences.rulerUnits;
	preferences.rulerUnits = Units.PIXELS;
	//store the current back and fore colour as we may be changing them
	obcol=backgroundColor;
	ofcol=foregroundColor;
	//load any stored preferences data
	getprefs()
	//call the user interface
	SetupUI()
	//restore the back and fore colour to the ones we stored earlier
	backgroundColor=obcol;
	foregroundColor=ofcol;
	//now restore the default unit preference
	preferences.rulerUnits = myunit;
	}	

/*
------------
FUNCTION: getprefs
------------
*/
function getprefs()
	{
	//set prefpath to point to the users Application Data folder, or local OS equivalent
	prefpath=Folder.userData;
	//set it to point to the vPSScriptPrefs folder within the users Application Data Folder
	prefpath=new Folder(prefpath+"/vPSScriptPrefs");
	//set preffile to point to the preferences file within the prefpath
	preffile=new File(prefpath+"/vBumpShadow_Prefs.jtp");
	//set previewimage and previewimageb to point to their respective images witin prefpath
	previewimage=new File(prefpath+"/vBumpShadow_Base.png");
	previewimageb=new File(prefpath+"/vBumpShadow_Swap.png");
		
	//check if the path exists
	if (prefpath.exists) 
		{
		//and if the preference file exists within it
		if (preffile.exists)
			{
			//open the file for edit and read the contents, 
			preffile.open("e");
			//copy this list into "presets"
			presets = preffile.read()
			//close the prefs file again
			preffile.close;
			//split the contents of the presets list into an array based on the ";" symbol
			presets=presets.split(";");
			getpreset(0);
			}
		// if no prefs file set prefs contents to "x" so we can check and create them later 
		//(We won't do it now in case the path test also comes up false)
		else {prefs='x';}
		//Also check if the preview image exists...
		if (previewimage.exists==false)
			{
			initpreview();
			}
		}
	//if the path didn't exist... 
	else 
		{
		//...create it
		prefpath.create();
		// since there cannot be any preference file set prefs contents to "x" so we can check and create them later
		prefs='x';
		}
	//check prefs for "x" meaning there was no preference data found and do the following if it's there.
	if (prefs=='x')
		{
		// Set up a string such as would be loaded if the prefs file existed
		// containing a basic presets list using some default information
		presets='last used,2.5,40,1.01,18;defaults,2.5,40,1.01,18';
		presets=presets.split(";");
		getpreset(0);
		}
	}

/*
------------
FUNCTION: getpreset
------------
*/
function getpreset(presetnum)
	{
	//set "prefs" to be the desired entry in the presets array
	prefs=presets[presetnum]
	//split the contents on any comma and place into "prefs" as another array
	prefs=prefs.split(",");
	//convert numeric strings to real numbers
	// since Levels Input Range End and Output Range End are set we don't need pref data for those  
	prefs[1]=parseFloat(prefs[1]); // highpass value
	prefs[2]=parseInt(prefs[2]); // levels Input Range Start
	prefs[3]=parseFloat(prefs[3]); // levels Gamma
	prefs[4]=parseInt(prefs[4]); // levels Output Range Start
	}

/*
------------
FUNCTION: makepresetlist
------------
*/
function makepresetlist()
	{
	//get the length of the current list of presets
	var numpresets=presets.length;
	//clear out the current entries in the listbox
	uipreset.removeAll();
	//create a loop incrementing from 0 until the end of the presets array
	for ( var i = 0; i < numpresets; i++ )
		{
		//make tempstore reference the current array entry
		var tempstore = presets[i];
		//split the current entry into an array in itself
		tempstore=tempstore.split(",");
		
		//grab the first entry in that array, which should be the name of the preset, and add it to the end of our listbox
		uipreset.add('item',tempstore[0]);		
		}
	//alert(presetlist.concat());
	}

	
/*
------------
FUNCTION: SetupUI
------------
*/
function SetupUI()
	{
	setupmainwindow =
	"dialog { text: 'vBumpShadow', orientation: 'row' \
		, controlpan: Panel { text: 'Options:', orientation: 'column' \
			, distancetext: StaticText {text: 'Shadow Distance:'} \
			, distancegroup: Group {orientation: 'row', alignment: 'left' \
				, distanceslider: Slider {value:6,minvalue:0.1,maxvalue:100,preferredSize:[100,20],alignment:'left'} \
				, distancenumber: EditText {text:'6',preferredSize:[50,20]} \
			}\
			, sharptext: StaticText {text: 'Sharpness:'} \
			, sharpgroup: Group {orientation: 'row', alignment: 'left' \
				, sharpslider: Slider {value:6,minvalue:0,maxvalue:126,preferredSize:[100,20],alignment:'left'} \
				, sharpnumber: EditText {text:'6',preferredSize:[50,20]} \
			}\
			, gammatext: StaticText {text: 'Gamma:'} \
			, gammagroup: Group {orientation: 'row', alignment: 'left' \
				, gammaslider: Slider {value:6,minvalue:0.10,maxvalue:9.99,preferredSize:[100,20],alignment:'left'} \
				, gammanumber: EditText {text:'6',preferredSize:[50,20]} \
			}\
			, brighttext: StaticText {text: 'Brightness:'} \
			, brightgroup: Group {orientation: 'row', alignment: 'left' \
				, brightslider: Slider {value:6,minvalue:0,maxvalue:255,preferredSize:[100,20],alignment:'left'} \
				, brightnumber: EditText {text:'6',preferredSize:[50,20]} \
			}\
		}\
		, rightgroup: Group {orientation: 'column', alignment: 'left' \
			, previewpan: Panel { text: 'Preview', orientation: 'column', alignment: 'center' \
				, previewimage: Image {bounds: 'x:0, y:0, width:64, height:128'} \
				, previewbut: Button { text:'Refresh',helpTip:'Update the preview to reflect current settings.'} \
				, previewres: Button { text:'Reset',helpTip:'Return preview to default state, with no effect from the settings.'} \
			}\
		}\
		, buttongroup: Group {orientation: 'column', alignment: 'left'\
			, runbut: Button { text:'GO'} \
			, canbut: Button { text:'Cancel'} \
			, pretext: StaticText {text:'Presets:'} \
			, savbut: Button { text:'Add',helpTip:'Add current settings to the list below.'} \
			, rembut: Button { text:'Remove',helpTip:'Remove current preset from the list below.'} \
			, presets: DropDownList {alignment:'left', preferredSize:[80,25], helpTip:'List of stored presets.', properties:{items:['George','Peppard']} } \
		}\
	}";
	
	InitUI()
	}

/*
------------
FUNCTION: InitUI
------------
*/	
function InitUI()
	{
	//initialise the windows
	mainwindow = new Window(setupmainwindow);
	mainwindow.text=mainwindow.text+" v"+VERSION;
	
	//set alias' for window objects
	uiprevimage = mainwindow.rightgroup.previewpan.previewimage;
	uiprevref = mainwindow.rightgroup.previewpan.previewbut;
	uiprevres = mainwindow.rightgroup.previewpan.previewres;
	uishad = mainwindow.controlpan.distancegroup.distanceslider;
	uishadnum = mainwindow.controlpan.distancegroup.distancenumber;
	uisharp = mainwindow.controlpan.sharpgroup.sharpslider;
	uisharpnum = mainwindow.controlpan.sharpgroup.sharpnumber;
	uigamma = mainwindow.controlpan.gammagroup.gammaslider;
	uigammanum = mainwindow.controlpan.gammagroup.gammanumber;
	uibright = mainwindow.controlpan.brightgroup.brightslider;
	uibrightnum = mainwindow.controlpan.brightgroup.brightnumber;
	uirun = mainwindow.buttongroup.runbut;
	uican = mainwindow.buttongroup.canbut;
	uisav = mainwindow.buttongroup.savbut;
	uirem = mainwindow.buttongroup.rembut;
	uipreset = mainwindow.buttongroup.presets;
	
	//set up the preview image
	uiprevimage.icon = previewimage;
	
	//clear the presets dropdownbox
	uipreset.removeAll()
	//copy the presets names into the dropdown presets list
	makepresetlist();
	//select the default preset in the dropdown box
	uipreset.selection=0;
	
	// copy preference info into sliders and text boxes
	uishad.value = prefs[1];
	uishadnum.text = prefs[1];
	uisharp.value = prefs[2];
	uisharpnum.text = prefs[2];
	uigamma.value = prefs[3];
	uigammanum.text = prefs[3];
	uibright.value = prefs[4];
	uibrightnum.text = prefs[4];
		
	CallUI()	
	}

/*
------------
FUNCTION: CallUI
------------
*/
function CallUI()
	{
	//set functionality of options
	
	// Shadow text box functionality
	uishadnum.onChange=function()
		{
		numboxeval(uishadnum,10,uishad,1,0.1,100);
		}
		
	// Sharpness text box functionality
	uisharpnum.onChange=function()
		{
		numboxeval(uisharpnum,1,uisharp,2,0,126);
		}
		
	// gamma text box functionality
	uigammanum.onChange=function()
		{
		numboxeval(uigammanum,100,uigamma,3,0.10,9.99);
		}
	
	// Brightness text box functionality
	uibrightnum.onChange=function()
		{
		numboxeval(uibrightnum,1,uibright,4,1,255);
		}
		
	// Shadow slider functionality
	uishad.onChanging=function()
		{
		slidereval(uishad,10,uishadnum,1);
		}
		
	// Sharpness slider functionality
	uisharp.onChanging=function()
		{
		slidereval(uisharp,1,uisharpnum,2);
		}		
	
	// Gamma slider functionality
	uigamma.onChanging=function()
		{
		slidereval(uigamma,100,uigammanum,3);
		}
		
	// Brightness slider functionality
	uibright.onChanging=function()
		{
		slidereval(uibright,1,uibrightnum,4);
		}
		
	// Refresh Preview Functionality
	uiprevref.onClick=function()
		{
		//copy the buffer image into the preview box
		uiprevimage.icon = previewimageb;
		//open the original preview image
		previmage = open(File(previewimageb));
		//run the shadowmagic function on the image
		shadowmagic(previmage,prefs[1],prefs[2],prefs[3],prefs[4]);
		//save and close the image
		previmage.saveAs(previewimage, PNGSaveOptions, false, Extension.NONE);
		previmage.close(SaveOptions.DONOTSAVECHANGES);
		//copy the original image back into the preview box
		uiprevimage.icon = previewimage;
		}
	
	// Reset Preview Button
	uiprevres.onClick=function()
		{
		uiprevimage.icon = previewimageb;
		initpreview();
		uiprevimage.icon = previewimage;
		}
		
	//remove preset button functionality
	uirem.onClick=function()
		{
		if (uipreset.selection.index==0||uipreset.selection.index==1)
			{
			alert("The current preset is protected and cannot be removed");
			return;
			}
		if (confirm("Are you sure you wish to remove the current preset?",true,"Remove Preset") == true)
			{
			//create a temporary storage array
			var tempstore=new Array();
			//get the length of the current list of presets
			var numpresets=presets.length;
			
			// get the currently selected preset index
			var removeme=uipreset.selection.index;
			//alert(removeme);
			//create a loop incrementing from 0 until the end of the presets array
			for ( var i = 0; i < numpresets; i++ )
				{
				//if the current preset being refered to isn't the one we want to remove, do this (else do nothing)
				if (i != removeme)
					{
					//add the currently referenced preset to tempstore
					tempstore[tempstore.length]=presets[i]
					}
				}
			//we should now have a duplicate of the presets list, minus the entry to be removed -
			//we can now copy this back into "presets", effectively removing the unwanted preset
			presets=tempstore;
			//open the preferences file and clear the contents
			preffile.open("w");
			//write that to file, connecting the array entries with a semi colon
			preffile.write(tempstore.join(";"));
			//close the file
			preffile.close();
			//remake the preset list in the presets dropdown box to remove the now nonexistant entry.
			makepresetlist();					
			}
		}
		
	//save preset button functionality
	uisav.onClick=function()
		{
		var prename=prompt("Please give the settings you wish to store a relevant name. A name with less than 12 characters is recommended.","My Settings","Preset Alias");
		// If the user clicked Cancel then just return (closing the window)
		if (prename==null) return;
		// Add prename to the presets box as an item
		uipreset.add('item',prename);
		// Add the current settings to the end of the presets array
		presets[presets.length]=prename+","+prefs[1]+","+prefs[2]+","+prefs[3]+","+prefs[4];
		//open the preferences file and clear the contents
		preffile.open("w");
		//copy presets to a temporary variable (paranoid, probably not required)
		var savepresets = presets;
		//write that to file, connecting the array entries with a semi colon
		preffile.write(savepresets.join(";"));
		//close the file
		preffile.close();
		}
	
	//preset selection functionality
	uipreset.onChange=function()
		{
		// get the index number of the newly selected preset 
		var getme = uipreset.selection.index;
		// run getpreset to get the preset that matches the selected index.
		getpreset(getme);
		//copy the preferences into the various controls:
		uishadnum.text = prefs[1];
		uisharpnum.text = prefs[2];
		uigammanum.text = prefs[3];
		uibrightnum.text = prefs[4];

		uishad.value = prefs[1];
		uisharp.value = prefs[2];
		uigamma.value = prefs[3];
		uibright.value = prefs[4];
		}
		
	// Go Button functionality
	//uirun.onClick=function(){alert("Bugger!")};
	uirun.onClick=function(){mainwindow.close(4)};
	uican.onClick=function(){mainwindow.close(2)};
		
	//open the UI
	mainwindow.center(); prefresult=mainwindow.show();
	if (prefresult==4)
		{
		//copy current prefs into first slot of the presets array
		presets[0]="last used,"+prefs[1]+","+prefs[2]+","+prefs[3]+","+prefs[4];
		//create prefs based on current values and save:
		preffile.open("w");
		preffile.write(presets.join(";"));
		preffile.close();
		
		
		// Clone the original image
		theduplicate=thepsd.duplicate()
		//set the default TGA colour depth
		TargaSaveOptions.resolution=TargaBitsPerPixels.TWENTYFOUR
		
		theduplicate.flatten();
		shadowmagic(theduplicate,prefs[1],prefs[2],prefs[3],prefs[4]);
		}
	}

/*
------------
FUNCTION: slidereval
------------
*/
function slidereval(input,decimal,output,prefnum)
	{
		//slider functionality
		// convert the input number into a useable length
		var commu = input.value;
		commu = commu * decimal;
		commu = Math.round(commu);
		commu /= decimal;
		// copy corrected number into box, slider and prefs array
		input.value=commu;
		output.text=commu;
		prefs[prefnum]=commu;
	}

/*
------------
FUNCTION: numboxeval
------------
*/
function numboxeval(input,decimal,output,prefnum,min,max)
	{
		//textbox functionality
		//set up temporary var and copy text input into it
		var tnum=input.text;
		//convert text into a floating point number
		tnum=parseFloat(tnum);
		// verify the input actually was a number
		if (isNaN(tnum)==true)
			{
			alert("Invalid Value!\nResetting to last valid number.");
			input.text=prefs[prefnum];
			}
		else
			{
			// clip the number to fit between the max and min amounts
			if (tnum < min) tnum = min;
			if (tnum > max) tnum = max;
			// then convert the number into a useable length
			var commu = tnum;
			commu = commu * decimal;
			commu = Math.round(commu);
			commu /= decimal;
			tnum = commu;
			//copy the converted value back into the box and into the prefs array and also into the associated slider bar.
			prefs[prefnum]=tnum;
			output.value=prefs[prefnum];
			input.text=prefs[prefnum];
			}	
	}

/*
------------
FUNCTION: initpreview
------------
*/
function initpreview()
	{
	// create the colours from which to create the preview
	var grey = new SolidColor();
	var white = new SolidColor();
	var black = new SolidColor();
	grey.rgb.hexValue="808080";
	white.rgb.hexValue="ffffff";
	black.rgb.hexValue="000000";
	
	//set the background colour to grey and then create a new document called "preview" using that colour
	backgroundColor=grey;
	previmage = app.documents.add(64,128,72,"preview",NewDocumentMode.RGB,DocumentFill.BACKGROUNDCOLOR);
	
	//create a square selection 8 pixels in from the top edge of the image
	var selshape = [[8,8],[8,56],[56,56],[56,8]];
	previmage.selection.select(selshape)
	
	//fill the selection with white before removing the selection
	previmage.selection.fill(white,ColorBlendMode.NORMAL,100)
	previmage.selection.deselect();
	
	//create a second square selection 8 pixels in from the bottom edge of the image
	var selshape = [[8,72],[8,120],[56,120],[56,72]];
	previmage.selection.select(selshape)

	//fill the selection with Black before removing the selection
	previmage.selection.fill(black,ColorBlendMode.NORMAL,100)
	previmage.selection.deselect();
	
	//Save this image as the preview icon then close
	previmage.saveAs(previewimage, PNGSaveOptions, false, Extension.NONE);
	previmage.saveAs(previewimageb, PNGSaveOptions, false, Extension.NONE);
	previmage.close(SaveOptions.DONOTSAVECHANGES);

	}	

/*
------------
FUNCTION: shadowmagic
------------
*/
function shadowmagic(input,shadow,sharpness,gamma,brightness)
	{
	
	//set mybackground to refer to the background layer (which should be the only layer)
	mybackground = input.artLayers[0];
	mybackground.desaturate();
	mybackground.applyHighPass(shadow/2.5);
	//adjust the images levels - blackpoint, whitepoint, gamma, max black, max white
	mybackground.adjustLevels(sharpness,128,gamma,brightness,255);
	}