// SCRIPT NAME: vCycleLayersets
var VERSION = 1.5;
// AUTHOR: James A. Taylor
// COPYRIGHT: James A. Taylor
// DESCRIPTION: Cycles visibility and selection of any layer sets named containing the ~ symbol


#target photoshop
app.bringToFront();

// GLOBAL VARIABLES
var wire,diff,spec,norm,thislayer,namelayer,backlayer,vislayers;

// START THE SCRIPT
main();


//FUNCTIONS

/*
------------
FUNCTION: main
------------
*/
function main()
	{
	// check if there are actually any documents open on which to perform the action
	if (documents.length <= 0)
		{
		return;
		}
	// set the variable "psdname" to the currently active documents name
	psdname=activeDocument.name;
	// set "thepsd" to point to the currently active document
	thepsd=activeDocument;
	
	if(thepsd.layers.length==1)
		{createtexturebase();}
	else
		{cyclelayers()}
	}
	
/*
------------
FUNCTION: createtexturebase
------------
*/
function createtexturebase()
	{
	//Better warn the user as to what's going to occur and let them quit:
	var conf=confirm("There is only one layer in the document!\n\nClicking \"Yes\" will assume that the document\nis a UVW layout map, and will set up the\nLayer groups accordingly.\n\nDo you wish to proceed?",false,"WARNING!");
	if (conf==false) return;
	thislayer=thepsd.activeLayer;
	backlayer=thepsd.layers[0];
	diff=thepsd.layerSets.add();
	spec=thepsd.layerSets.add();
	norm=thepsd.layerSets.add();
	diff.name="~ Diffuse";
	spec.name="~ Specular";
	norm.name="~ Normal ~";
	wire=thepsd.layerSets.add();
	wire.name="UV's";
	thislayer=backlayer;
	backlayer.duplicate(wire,ElementPlacement.INSIDE);
	thislayer=wire.artLayers[0];
	thislayer.name="UV's Solid";
	thislayer.blendMode=BlendMode.SCREEN;
	thislayer.duplicate(thislayer,ElementPlacement.PLACEAFTER);
	thislayer=wire.artLayers[1];
	thislayer.name="UV's Faint";
	thislayer.opacity=10.0;
	thislayer=backlayer;
	}

/*
------------
FUNCTION: cyclelayers
------------
*/
function cyclelayers()
	{
	thislayer=thepsd.activeLayer;
	// set up a storage array for the tidle marked layersets
	var tlayers = new Array();
	//set up an array of those T layersets that are currently visible
	var vlayers = new Array();
	var vlayer = 0;
	var i=(thepsd.layerSets.length-1);
	// loop until the count of i is the same as the number of layersets
	while (i> -1)
		{
		// define the current layerset (as number i) as "namelayer"
		var namelayer=thepsd.layerSets[i];
		// search the layers name for the "~" symbol
		var tiset = namelayer.name.indexOf("~");
		// if the ~ symbol was found...
		if (tiset >= 0)
			{
			//set the last entry in the tlayers to be i (the current layerset index)
			tlayers[tlayers.length]=i
			//if the layerset was currently visible...
			if (namelayer.visible==true)
				{
				// add 1 to vlayer and add the layersets index (i) to the vlayers array
				vlayer++
				vlayers[vlayers.length]=i
				}
			}
		// reduce i by 1
		i--
		}
		
	// if there are no layers marked with a ~ then return
	if (tlayers.length<1) return;
	//if the value of vlayer is more than 1 (so there are multiple ~ sets visible)...
	if (vlayer>1)
		{
		i=vlayer-1;
		while (i> -1)
			{
			var visi=vlayers[i];
			namelayer=thepsd.layerSets[visi];
			//alert(namelayer);
			namelayer.visible=false;
			i--;
			}
		// set the first listed ~ layerset to be visible
		thepsd.layerSets[tlayers[0]].visible=true;
		thepsd.activeLayer=thepsd.layerSets[tlayers[0]]
		}
	// if all ~ layersets are hidden, set the first to be visible
	if (vlayer==0)
		{
		thepsd.layerSets[tlayers[0]].visible=true;
		thepsd.activeLayer=thepsd.layerSets[tlayers[0]]
		}
	// if only one ~ layerset is visible...
	if (vlayer==1)
		{
		// convert the array "tlayers" into a string of numbers called "tlist"
		var tlist=tlayers.join("");
		// Search tlist for the visible  ~ layerset number, return it's location to visi
		var visi=tlist.indexOf(vlayers[0]);
		// hide the currently visible ~ layerset
		thepsd.layerSets[vlayers[0]].visible=false;
		// if the visible layer isn't the last in the list...
		if (visi<tlayers.length-1)
			{
			// show the layer marked by the visi index (the formerly visible layer) +1 (the next layer in the sequence).
			thepsd.layerSets[tlayers[visi+1]].visible=true;
			thepsd.activeLayer=thepsd.layerSets[tlayers[visi+1]];
			}
		else
			{
			// show the first layer in the sequence.
			thepsd.layerSets[tlayers[0]].visible=true;
			thepsd.activeLayer=thepsd.layerSets[tlayers[0]];
			}
		}
	}
