// SCRIPT NAME: vHexMagnet
var VERSION=1.34;
// AUTHOR: James A. Taylor
// COPYRIGHT: James A. Taylor
// DESCRIPTION: Creates a hexagonal tile pattern.


#target photoshop
app.bringToFront();

// GLOBAL VARIABLES
var myunit, obcol, ofcol, heximage, hexwidth, centrey, upx, dnx, hexagon,hexheight;
var thiscolor, colorlist, white, red, yellow, green, cyan, blue, purple, firstcolor;
var tileage, tiling, tilestate, hexlayer, colcount, rowcount;

// START THE SCRIPT
main();


//FUNCTIONS

/*
------------
FUNCTION: main
------------
*/
function main()
	{
	//store the default unit type and set the current preference to be pixels
	myunit = preferences.rulerUnits;
	preferences.rulerUnits = Units.PIXELS;
	//store the current back and fore colour as we'll be changing them
	obcol=backgroundColor;
	ofcol=foregroundColor;
	//call the user interface
	UI();
	//makehexagon()
	//restore the back and fore colour to the ones we stored earlier
	backgroundColor=obcol;
	foregroundColor=ofcol;
	//now restore the default unit preference
	preferences.rulerUnits = myunit;
	}

/*
------------
FUNCTION: UI
------------
*/
function UI()
	{
	setupui =
		"dialog {text: 'vHexMagnet', orientation:'column' \
			, stylegroup: Group {orientation:'row', alignment:'left'\
				, styletext: StaticText {text:'Style:'}\
				, style: DropDownList {properties:{items:['Primary Colour','Shaded Colour','Greyscale','Mono','From Image','Grid']} }\
			}\
			, hexsizegroup: Group {orientation:'row', alignment:'left' \
				, hsizetext: StaticText {text:'Hexagon Width:'} \
				, hsize: DropDownList {properties:{items:['4','8','16','32','64','128']}} \
			}\
			, resizegroup: Group {orientation: 'row', alignment:'left' \
				, restext: StaticText {text:'Image Size:'} \
				, sizex: DropDownList {properties:{items:['64','128','256','512','1024','2048']} } \
				, restext: StaticText {text:'x'} \
				, sizey: DropDownList {properties:{items:['64','128','256','512','1024','2048']} } \
			}\
			, tilegroup: Group {orientation: 'stack', alignment:'left' \
				, htiletext: StaticText {text:'Widths less than 16 will tile vertically by default', alignment:'left'} \
				, usetile: Checkbox {text:'Tile Vertically (distorts hexagons)', alignment:'left', value:true} \
			}\
			, hexcount: StaticText {text:'Hex Count: ', alignment:'right', preferredSize:[100,20]} \
			, progbar: Progressbar{preferredSize:[180,20]} \
			, btngroup: Group { orientation:'row', alignment:'center', alignclildren:'center' \
				, okbtn: Button { text:'OK', properties:{name:'OK'} } \
				, clbtn: Button { text:'Cancel'} \
			}\
		}";
	
	//initialise window
	mrwindow=new Window(setupui);
	//setup window UI reference variables
	uistyle=mrwindow.stylegroup.style;
	uihsize=mrwindow.hexsizegroup.hsize;
	uisizex=mrwindow.resizegroup.sizex;
	uisizey=mrwindow.resizegroup.sizey;
	uitile=mrwindow.tilegroup.usetile;
	uittext=mrwindow.tilegroup.htiletext;
	
	uok=mrwindow.btngroup.okbtn;
	ucan=mrwindow.btngroup.clbtn;
	uiprog=mrwindow.progbar;
	counter=mrwindow.hexcount;
	mrwindow.text=mrwindow.text+" v"+VERSION;
	
	//get the preferences data, or the defaults if no prefs exist
	getprefs();

	uistyle.selection=style;
	uihsize.selection=hexwidth;
	uisizex.selection=isizex;
	uisizey.selection=isizey;
	uitile.value=tileage;
	//left over from something I was trying that might come in useful again:
	uittext.visible=false;
	
	// set up size arrays
	ahsize=new Array(4,8,16,32,64,128);
	aisize=new Array(64,128,256,512,1024,2048);
	
	initializevalues();
	
	uistyle.onChange=function()
		{
		style=uistyle.selection.index;
		}

	//Set hexsize functionality
	uihsize.onChange=function()
		{
		hexwidth=ahsize[uihsize.selection.index];
		calchex();
		}
	//Set Image Size Functonality
	uisizex.onChange=function()
		{
		isizex=aisize[uisizex.selection.index];
		calchex();
		}
	uisizey.onChange=function()
		{
		isizey=aisize[uisizey.selection.index];
		calchex();
		}
	//Set checkbox functionality
	uitile.onClick=function() 
		{
		(tileage!=true)?tileage=true:tileage=false;
		calchex();
		}
		
	uok.onClick=function()
		{
		//lock the OK button so it can't be clicked again
		uok.onClick=function(){}
		
		//save the current settings to the preferences
		saveprefs();
		
		//run the script
		setupimage();
		makehexagon();
		if (style!=4) colorset();
		makegrid();
		
		//clear selection
		hexagon.deselect();
		
		// close the window
		mrwindow.close(4);
		}
	ucan.onClick=function(){mrwindow.close(2)};

	//open window
	mrwindow.center(); result=mrwindow.show();
	}

/*
------------
FUNCTION: initializevalues
------------
*/
function initializevalues()
	{
	style=uistyle.selection.index;
	hexwidth=ahsize[uihsize.selection.index];
	isizex=aisize[uisizex.selection.index];
	isizey=aisize[uisizey.selection.index];
	calchex();
	}

/*
------------
FUNCTION: setupimage
------------
*/
function setupimage()
	{
	if (style==4)
		{
		//if we're working on an existing image, duplicate, flatten and resize it
		heximage=activeDocument.duplicate();
		heximage.flatten();	
		hexlayer=heximage.activeLayer;
		heximage.resizeImage(isizex,isizey,72.00);
		}
	else
		{
		// make new document called "hexagon", filled with white
		app.documents.add(isizex,isizey,72,"hexagons",NewDocumentMode.RGB,DocumentFill.WHITE);
		// let that new doc be know as heximage
		heximage=activeDocument;
		//add a blank layer and call it "hexagon"
		heximage.artLayers.add();
		heximage.layers[0].name="hexagon";
		}
	}

/*
------------
FUNCTION: makehexagon
------------
*/
function makehexagon()
	{
	// set up a selection shape array 
	var selshape = [[centrey,0],[hexwidth,upx],[hexwidth,dnx],[centrey,hexheight],[0,dnx],[0,upx]];
	// Set the selection as the defined shape
	heximage.selection.select(selshape);
	hexagon=heximage.selection;
	
	//adjust the border on the selection to avoid transparency in the final result:
	quickmask(true);
	leveladjust();
	quickmask(false);
	
	//move the selection to it's starting posiition
	hexagon.translateBoundary(0,-(hexheight/2));
	return;
	}
	
/*
------------
FUNCTION: colorset
------------
*/
function colorset()
	{
	//set up some solid colours
	//there's probably a really elegant way to do this, with loops and sub arrays and so on.
	//stuff that though, I'm too lazy to work it out right now.
	
	white=new SolidColor();
	white.rgb.hexValue="ffffff";
	red=new SolidColor();
	red.rgb.hexValue="ff0000";
	yellow=new SolidColor();
	yellow.rgb.hexValue="ffff00";
	green=new SolidColor();
	green.rgb.hexValue="00ff00";
	cyan=new SolidColor();
	cyan.rgb.hexValue="00ffff";
	blue=new SolidColor();
	blue.rgb.hexValue="0000ff";
	purple=new SolidColor();
	purple.rgb.hexValue="ff00ff";
	black=new SolidColor();
	black.rgb.hexValue="000000";
		
	if (style==0) colorlist=new Array(red,yellow,green,cyan,blue,purple);
	if (style==3) colorlist=new Array(white,black);
	if (style==5) colorlist=new Array(black)
	}

/*
------------
FUNCTION: randomcolor
Properties: enter-whether to enter the quickmask or exit it (boolean)
------------
*/
function randomcolor()
	{
	if (style==1)
		{
		//generate a random color
		var tempcolor=new SolidColor();
		tempcolor.rgb.red=Math.floor(Math.random()*256);
		tempcolor.rgb.green=Math.floor(Math.random()*256);
		tempcolor.rgb.blue=Math.floor(Math.random()*256);
		thiscolor=tempcolor;
		}
	if (style==2)
		{
		//generate a shade of grey
		var grey = Math.floor(Math.random()*256);
		var tempcolor = new SolidColor();
		tempcolor.rgb.red=grey;
		tempcolor.rgb.green=grey;
		tempcolor.rgb.blue=grey;
		thiscolor=tempcolor;
		}
	}

/*
------------
FUNCTION: calchex
Calculates the number of hexagons in a row, and in a column, and then the number in the image.
------------
*/
function calchex()
	{
	//get the pixel sizes we'll need for creating the hexagon
	
	//if we want the hexagons to tile they'll need to be distorted vertically
	//otherwise they can be the same size vertically as horizontal (which looks nicer, but can't tile vertically)
	(tileage==true) ? hexheight=(hexwidth+(hexwidth/3)) : hexheight=hexwidth;
	//find the horizontal center
	centrey=Math.round(hexwidth/2);
	//find the vertical quarters 
	upx=hexheight/4;
	dnx=hexheight-upx;
	
	// set up the number of columns
	colcount=(Math.floor(isizex/hexwidth));
	//set up the rows based on the size of the hexagon
	rowcount=Math.round(isizey/(hexheight-upx))+1
	
	hexcount=(colcount*rowcount)+rowcount/2;
	if (tileage==true) hexcount=hexcount-0.5;
	counter.text='Hex Count: '+hexcount;
	}

/*
------------
FUNCTION: makegrid
------------
*/
function makegrid()
	{
	// set up columns loop variable
	var i=0;
	// set up rows loop variable
	var j=0;

	// set up a variable so we can tell if the row is odd or even
	var odd;
	//set up the maximum value for the progress bar based on the number of rows.
	uiprog.maxvalue=(rowcount+(colcount*rowcount));
	//set up an array to store the colour values of the top row for translation to the bottom one
	var rowcolors=new Array();
	var countdown=0;
	
	//fill the screen with hexagon goodness
	while(j!=rowcount)
		{
		//find out if j is odd or even. the var even will be true if it is.
		odd=j%2;
		//do a row
		while(i<colcount)
			{
			starttime=new Date();
			startsec=starttime.getMinutes();		
			//the if is to stop the selection exiting the image at the end of the row, causing a crash
			if (i!==0)
				{
				//move the selection along one hex position
				hexagon.translateBoundary(hexwidth,0);
				}
			if (style==4)
				{
				hexlayer.applyAverage();
				}
			else
				{
				if (j<rowcount-1)
					{
					if (style==1||style==2)
						{
						randomcolor()
						}
					else
						{
						//select a random colour from the array
						thiscolor=colorlist[Math.floor(Math.random()*(colorlist.length))];
						}
					// if this is the first hex on this row and it's an even one, store the color
					if ((i==0&&odd==1)) firstcolor=thiscolor;
					
					// if it's also the first row then store the color in the color array
					if (j==0) rowcolors[rowcolors.length]=thiscolor;
					// if it's the last hex on the row and it's even, then replace the random color 
					// with the stored one so the hexagons match for tiling
					if ((i==colcount-1)&&(odd==1)) thiscolor=firstcolor;
					}
				else
					{
					//add an extra color since if we're not tiling there won't be a 9th color to use in the array
					//I could make it so not tiling means the collars and cuffs don't match, but I'm lazy.
					if (i>(rowcolors.length-1)) rowcolors[rowcolors.length]=colorlist[Math.floor(Math.random()*(colorlist.length))];
					thiscolor=rowcolors[i];
					}
				if (style==0||style==1||style==2||style==3)
					{
					hexagon.fill(thiscolor,ColorBlendMode.NORMAL,100);
					}
				else
					{
					hexagon.stroke(thiscolor,1,StrokeLocation.CENTER,ColorBlendMode.NORMAL,100);
					}
				}

			//update progressbar & information
			uiprog.value++
			countdown++
			hexleft=hexcount-countdown;
			counter.text='Hex Count: '+(hexleft);
			i++
			}

		//work out how much to move the selection based on j being odd or even
		if(odd==1)
			{
			colcount=(Math.floor(heximage.width.value/hexwidth));
			hexagon.translateBoundary(-(heximage.width.value)+centrey,hexheight-upx); 
			}
			else
			{
			colcount=(heximage.width.value/hexwidth)+1
			hexagon.translateBoundary(-((hexwidth*colcount)-hexwidth)+centrey,hexheight-upx);
			}
		//alert('row '+j+' of '+rowcount);
		j++;
		
		//update progressbar
		//uiprog.value++;
		i=0;
		}
	}
	
/*
------------
FUNCTION: getprefs
> Checks to see if the path and file for the preferences exists, and if they do loads them into a variable.
------------
*/
function getprefs()
	{
	//set default values	
	style=0;
	hexwidth=3;
	isizex=2;
	isizey=2;
	tileage = true;
	
	//set prefpath to point to the users Application Data folder, or local OS equivalent
	prefpath=Folder.userData;
	//set it to point to the vPSScriptPrefs folder within the users Application Data Folder
	prefpath=new Folder(prefpath+"/vPSScriptPrefs");
	//set preffile to point to the preferences file within the prefpath 
	preffile=new File(prefpath+"/vhexmagnet_prefs.jtp");
	//check if the path exists
	if (prefpath.exists) 
		{
		if (preffile.exists)
			{
			//open the preference file
			preffile.open("r");
			prefs=preffile.read()
			eval(prefs);
			preffile.close();
			}
		}
	//if the path didn't exist, create it
	else 
		{
		prefpath.create();
		}
	}

/*
------------
FUNCTION: saveprefs
> Checks to see if the path and file for the preferences exists, and if they do loads them into a variable.
------------
*/
function saveprefs()
	{
	//copy the pref variables into saveprefs
	//note we're using the variable fullstring instead of ecmdline, but when reloaded that information will be processed as ecmdline.
	var saveprefs='style = ' +uistyle.selection+ ';\nhexwidth = ' +uihsize.selection+ ';\nisizex = ' +uisizex.selection+ ';\nisizey = ' +uisizey.selection+ ';\ntileage = ' +tileage+ ';';

	//then open the file, destroying the contents,
	preffile.open("w");
	//then write the prefs to the file and close it.
	preffile.write(saveprefs);
	preffile.close();
	}



/*
------------
FUNCTION: quickmask
Properties: enter-whether to enter the quickmask or exit it (boolean)
------------
*/
function quickmask(enter)
	{
	//based on "enter" set the variable to create or clear the mask.
	if (enter==true)
		{var id4223 = charIDToTypeID( "setd" );}
	else
		{var id4223 = charIDToTypeID( "Cler" );}
		
    var desc923 = new ActionDescriptor();
    var id4224 = charIDToTypeID( "null" );
    var ref536 = new ActionReference();
    var id4225 = charIDToTypeID( "Prpr" );
    var id4226 = charIDToTypeID( "QucM" );
    ref536.putProperty( id4225, id4226 );
    var id4227 = charIDToTypeID( "Dcmn" );
    var id4228 = charIDToTypeID( "Ordn" );
    var id4229 = charIDToTypeID( "Trgt" );
    ref536.putEnumerated( id4227, id4228, id4229 );
    desc923.putReference( id4224, ref536 );
	executeAction( id4223, desc923, DialogModes.NO );
	}
	
/*
------------
FUNCTION: leveladjust
------------
*/
function leveladjust()
	{
	//we only need to adjust the levels on the lower half of the selection
	//because the antialiasing at the top of the next row will blend with the hard edge
	//having top and bottom antialiased causes holes in the transparency
	//while having both withough looks ugly.
	
	//select the bottom of the quickmask selection
	
	// set up a selection shape array 
	var levshape = [[0,dnx],[hexwidth,dnx],[hexwidth,hexheight],[0,hexheight]];
	// Set the selection as the defined shape
	heximage.selection.select(levshape);
	var tempselect=heximage.selection;
	
	// Adjust Levels
	var id4230 = charIDToTypeID( "Lvls" );
    var desc924 = new ActionDescriptor();
    var id4231 = charIDToTypeID( "Adjs" );
    var list126 = new ActionList();
    var desc925 = new ActionDescriptor();
    var id4232 = charIDToTypeID( "Chnl" );
    var ref537 = new ActionReference();
    var id4233 = charIDToTypeID( "Chnl" );
    var id4234 = charIDToTypeID( "Ordn" );
    var id4235 = charIDToTypeID( "Trgt" );
    ref537.putEnumerated( id4233, id4234, id4235 );
    desc925.putReference( id4232, ref537 );
    var id4236 = charIDToTypeID( "Inpt" );
    var list127 = new ActionList();
    list127.putInteger( 0 );
    list127.putInteger( 2 );
    desc925.putList( id4236, list127 );
    var id4237 = charIDToTypeID( "LvlA" );
    list126.putObject( id4237, desc925 );
    desc924.putList( id4231, list126 );
	executeAction( id4230, desc924, DialogModes.NO );
	}
	