// SCRIPT NAME: vPlasma
var VERSION=1.43;
// AUTHOR: James A. Taylor
// COPYRIGHT: James A. Taylor
// DESCRIPTION: "Like the clouds filter on steroids" - Script to create semi-random multi-colour cloud like images.

#target photoshop
app.bringToFront();

//GLOBAL VARIABLES
var obcol,ofcol;
var docx=512,docy=512,maxit=100,prog,contrast;
var rancyc, cycles, filltypes, posix, posiy, cpos=0;
var maxdist,gcoordx1,gcoordy1,gcoordx2,gcoordy2,tmp;
// preference variables
var prefs,preffile,prefpath;

/*
PREFERENCE ORDER
0	Version
1	xsize
2	ysize
3	# cycles
4	Randomise cycles (True/False)
5	Plasma type (0 through 5)
6	Greyscale result (true/false)
7	Contrast (0 to 100)	
*/

//START THE SCRIPT
main();

//FUNCTIONS

/*
------------
FUNCTION: main
------------
*/
function main()
	{
	//store the default unit type and set the current preference to be pixels
	myunit = preferences.rulerUnits;
	preferences.rulerUnits = Units.PIXELS;
	//store the current back and fore colour as we'll be changing them
	obcol=backgroundColor;
	ofcol=foregroundColor;
	// pretty simple, just call the Prefs then UI
	checkprefs()
	UI()
	//restore the back and fore colour to the ones we stored earlier
	backgroundColor=obcol;
	foregroundColor=ofcol;
	//now restore the default unit preference
	preferences.rulerUnits = myunit;
	}

/*
------------
FUNCTION: checkprefs
------------
*/
function checkprefs()
	{
	//set prefpath to point to the users Application Data folder, or local OS equivalent
	prefpath=Folder.userData;
	//set it to point to the vPSScriptPrefs folder within the users Application Data Folder
	prefpath=new Folder(prefpath+"/vPSScriptPrefs");
	//set preffile to point to the preferences file within the prefpath 
	preffile=new File(prefpath+"/vplasma_prefs.jtp");
	//check if the path exists
	if (prefpath.exists) 
		{
		if (preffile.exists)
			{
			preffile.open("e");
			prefs=preffile.read()
			prefs=prefs.split(",");
			//convert boolean strings in Prefs to actual booleans
			(prefs[4]=='true') ? prefs[4]=true:prefs[4]=false;
			(prefs[6]=='true') ? prefs[6]=true:prefs[6]=false;
			//convert numeric values to real numbers
			prefs[0]=parseInt(prefs[0]);
			prefs[1]=parseInt(prefs[1]);
			prefs[2]=parseInt(prefs[2]);
			prefs[3]=parseInt(prefs[3]);
			prefs[5]=parseInt(prefs[5]);
			prefs[7]=parseInt(prefs[7]);
			preffile.close;
			}
		else {prefs='x';}
		}
	//if the path didn't exist, create it
	else 
		{
		prefpath.create();
		prefs='x';
		}
	}



/*
------------
FUNCTION: UI
------------
*/
function UI()
	{
	setupui =
		"dialog {text: 'vPlasma', orientation:'column' \
			, resizepan: Panel {text:'Image Size:',orientation: 'column', alignment:'left', preferredSize:[235,75] \
				, resizegroup: Group {orientation: 'row', alignment:'center' \
					, sizex: EditText {text:'512',preferredSize:[50,20]} \
					, restext: StaticText {text:'x'} \
					, sizey: EditText {text:'512',preferredSize:[50,20]} \
				}\
				, usecur: Button {text:'Get From Selected Image', alignment: 'center'} \
			}\
			, itts: Panel {text: 'Plasma', orientation:'column', alignment:'left',preferredSize:[235,75] \
				, ittsg: Group {orientation:'row',alignment:'left' \
					, ittt: StaticText {text: 'Cycles'} \
					, ittss: Slider {value:100,minvalue:0,maxvalue:100,preferredSize:[100,20],alignment:'center'} \
					, ittsv: StaticText {text: '100',preferredSize:[35,20],alignment:'center'} \
				}\
				, itttg: Group {orientation:'row', alignment:'left' \
					, ittt: StaticText {text:'Type'} \
					, ittd: DropDownList {alignment:'left', properties:{items:['Cloud','Gradient','Radial','Angle','Line','Bubble','Diamond','Spin','Twist']} } \
				}\
				, ittcong: Group {orientation:'row',alignment:'left' \
					, ittcont: StaticText {text: 'Contrast'} \
					, ittcons: Slider {value:0,minvalue:0,maxvalue:100,preferredSize:[100,20],alignment:'center'} \
					, ittconv: StaticText {text: '0',preferredSize:[35,20],alignment:'center'} \
				}\
				, ittc: Checkbox {value:false,text:'Randomise Cycles', alignment:'left'} \
				, greys: Checkbox {value:false,text:'Greyscale Result', alignment:'left'} \
			}\
			,progbar: Progressbar{preferredSize:[180,20]} \
			, btngroup: Group { orientation:'row', alignment:'center', alignclildren:'center' \
				, okbtn: Button { text:'OK', properties:{name:'OK'} } \
				, clbtn: Button { text:'Cancel'} \
			}\
		}";
		
	//initialise window
	mrwindow=new Window(setupui);
	
	//setup window UI reference variables
	uiits=mrwindow.itts.ittsg.ittss;
	uiitv=mrwindow.itts.ittsg.ittsv;
	uiitc=mrwindow.itts.ittc;
	uiitt=mrwindow.itts.itttg.ittd;
	contrast=mrwindow.itts.ittcong.ittcons;
	conval=mrwindow.itts.ittcong.ittconv;
	uisizex=mrwindow.resizepan.resizegroup.sizex;
	uisizey=mrwindow.resizepan.resizegroup.sizey;
	uicursize=mrwindow.resizepan.usecur;
	uigrey=mrwindow.itts.greys;
	uiprog=mrwindow.progbar;
	uok=mrwindow.btngroup.okbtn;
	ucan=mrwindow.btngroup.clbtn;
	mrwindow.text=mrwindow.text+" v"+VERSION;
	
	//Set initial variable values
	uiprog.value=0;
	
	if (prefs=='x')
		{
		uiits.value=25;
		uiitv.text="25";
		uiitt.selection=1;
		}
	else
		{
		uisizex.text=prefs[1];
		uisizey.text=prefs[2];
		uiits.value=prefs[3];
		uiitv.text=uiits.value;
		uiitc.value=prefs[4];
		uiitt.selection=prefs[5];
		uigrey.value=prefs[6];
		contrast.value=prefs[7];
		conval.text=contrast.value;
		}
		// Store size values in case of problems
		var sizex=parseInt(uisizex.text);
		var sizey=parseInt(uisizey.text);
	
	//set slider functionality
	uiits.onChanging=function()
		{
		uiitv.text=Math.round(uiits.value);
		}
	contrast.onChanging=function()
		{
		conval.text=Math.round(contrast.value);
		}
	//set use size button functionality
	uicursize.onClick=function()
		{
		// check if there are actually any documents open on which to perform the action
		if (documents.length <= 0)
			{
			return;
			}
		else
			{
			// copy the information from the currently selected image
			var thesource = activeDocument;
			uisizex.text=parseInt(thesource.width.toString());
			uisizey.text=parseInt(thesource.height.toString());
			sizex=parseInt(uisizex.text);
			sizey=parseInt(uisizey.text);
			}
	 	}
	 	
	//Check Text Box values are valid while changing
	uisizex.onChanging=function()
		{
		//make tnum a number derived from the string in the textbox
		var xnum=parseInt(uisizex.text);
		//convert this back to a string so we can work out how many characters it contains
		var xnum2=xnum.toString();
		//If tnum was NaN, or if tnum2's length was less than the number of characters in the box do this 
		if ((xnum2.length<uisizex.text.length)||(isNaN(xnum)==true))
			{
			uisizex.text=sizex;
			}
		else
			{
			sizex=parseInt(uisizex.text);
			}
		}	
	// do the same for the Y value			
	uisizey.onChanging=function()
		{
		var ynum=parseInt(uisizey.text);
		var ynum2=ynum.toString();
		if ((ynum2.length<uisizey.text.length)||(isNaN(ynum)==true))
			{
			uisizey.text=sizey;
			}
		else
			{
			sizey=parseInt(uisizey.text);
			}
		}
		
	//Set Button Functionality (copy values to useable vars)
	uok.onClick=function()
		{
		//lock the OK button so it can't be clicked again
		uok.onClick=function(){}
		docx=sizex;
		docy=sizey;
		rancyc=uiitc.value;
		maxit=Math.round(uiits.value);
		//create prefs based on current values and save:
		var outprefs=new Array(VERSION,docx,docy,maxit,rancyc,uiitt.selection.index,uigrey.value,contrast.value);
		preffile.open("w");
		preffile.write(outprefs.concat());
		preffile.close();
		//set base maximum for progress bar
		uiprog.maxvalue=maxit*3;
		//run the script
		newdoc();
		// close the window
		mrwindow.close(4);
		}
	ucan.onClick=function(){mrwindow.close(2)};
	
	//open window
	mrwindow.center(); result=mrwindow.show();
	//if(result==4) newdoc()
		
	}	
	
	
/*
------------
FUNCTION: newdoc
------------
*/
function newdoc()
	{
	app.documents.add(docx,docy,72,"plasma",NewDocumentMode.RGB,DocumentFill.WHITE);
	// let that new doc be know as mrplasma
	mrplasma=activeDocument;
	//freeze th ehistory for the document so we don't just get hundreds of gradient fills or something - while frozen, do the plasma function.
	mrplasma.suspendHistory("vPlasma","plasma();");
	}

/*
------------
FUNCTION: plasma
------------
*/
function plasma()
	{
	// set up gradient fill type
	filltypes=new Array("None","Rflc","Rdl ","Angl","Lnr ","Rdl ","Angl","Rdl ","Dmnd");
	filltypes=filltypes[uiitt.selection.index]; 
	//set up channels
	var chans=new Array("Rd  ","Grn ","Bl  ","RGB ");
	//set up divisions for "Diamond"
	//posix=new Array(0,docx/4,docx/2,(docx/2)+(docx/4),docx);
	//posiy=new Array(0,docy/4,docy/2,(docy/2)+(docy/4),docy);
	posix=new Array(0,docx/2,docx);
	posiy=new Array(0,docy/2,docy);

	var i=0
	while (i<4)
		{
		if (rancyc==true)
			{
			// randomise number of cycles up to set maximum
			cycles = Math.random()*maxit;
			cycles = Math.ceil(cycles);
			// adjust progress bar maximum to account for change
			uiprog.maxvalue=uiprog.maxvalue-maxit+cycles;
			}
		else
			{
			cycles = maxit;
			//uiprog.maxvalue=maxit*3;;
			}
		// if we're doing bubbles we should massage the number of cycles for a good result
		if (uiitt.selection.index==5) 
			{
			cycles=cycles*2;
			if (cycles%2==0) cycles=cycles+1;
			// adjust progress bar maximum to account for change
			uiprog.maxvalue=uiprog.value+cycles;
			}
		
		//===SELECT CHANNEL===
		var id108 = charIDToTypeID( "slct" );
		    var desc19 = new ActionDescriptor();
		    var id109 = charIDToTypeID( "null" );
		        var ref7 = new ActionReference();
		        var id110 = charIDToTypeID( "Chnl" );
		        var id111 = charIDToTypeID( "Chnl" );
		        var id112 = charIDToTypeID( chans[i] );
		        ref7.putEnumerated( id110, id111, id112 );
		    desc19.putReference( id109, ref7 );
		executeAction( id108, desc19, DialogModes.NO );
		// if the loop number is less than 3 we're still working on the channels
		// over three and the composite is selected and we can end 
		if (i<3) 
			{
			// if the type is anything but cloud we're using a gradient
			if (uiitt.selection.index>0)
				{
				gradients(cycles);
				}
			else
				{
				//alert('cloud!');
				cloud(cycles);
				//do a final auto contrast per channel
				var id20 = charIDToTypeID( "Lvls" );
			    var desc3 = new ActionDescriptor();
			    var id21 = charIDToTypeID( "AuCo" );
			    desc3.putBoolean( id21, true );
				executeAction( id20, desc3, DialogModes.NO );
				}
			//adjust the contrast for the channel
			mrplasma.activeLayer.adjustBrightnessContrast(0,contrast.value);
			}
		i++
		}
	if (uigrey.value==true)
		{
		// desaturate image
		var id179 = charIDToTypeID( "Dstt" );
		executeAction( id179, undefined, DialogModes.NO );
		}
	}

/*
------------
FUNCTION: cloud
------------
*/
function cloud(cycles)
	{
	var i=0
	while (i<cycles)
		{
		i++;
		//increment progress bar
		uiprog.value=uiprog.value+1;
		//do a random throw
		var lev = Math.random();
		//run difference cloud filter
		var id18 = charIDToTypeID( "DfrC" );
		executeAction( id18, undefined, DialogModes.NO );
		
		//check random throw to see whether to call an equalise or auto contrast
		if (lev<=.1)
			{
			//do an equalise
			var id19 = charIDToTypeID( "Eqlz" );
			executeAction( id19, undefined, DialogModes.NO );
			}
		if (lev>=.9)
			{		
			//do auto contrast
			var id20 = charIDToTypeID( "Lvls" );
		    var desc3 = new ActionDescriptor();
		    var id21 = charIDToTypeID( "AuCo" );
		    desc3.putBoolean( id21, true );
			executeAction( id20, desc3, DialogModes.NO );
			}
		}
	}


/*
------------
FUNCTION: gradients
------------
*/
function gradients(cycles)
	{
		var i=0
		while (i<cycles)
		{
		i++;
		//increment progress bar
		uiprog.value=uiprog.value+1;
		// if the selection is not "Bubble" do all this:
		if (uiitt.selection.index<=4)
			{
			// calculate some random coordinates based on the document size
			gcoordx1 = Math.random()*(docx-1);
			gcoordx1 = Math.ceil(gcoordx1);
			gcoordy1 = Math.random()*(docy-1);
			gcoordy1 = Math.ceil(gcoordy1);
			// if the type is anything other than "line" do this
			if (uiitt.selection.index<4)
				{
				gcoordx2 = Math.random()*(docx-1);
				gcoordx2 = Math.ceil(gcoordx2);
				gcoordy2 = Math.random()*(docy-1);
				gcoordy2 = Math.ceil(gcoordy2);
				}
			// if it is "line" do this
			if (uiitt.selection.index==4)
				{
				//random throw from 1 to 20
				gcoordx2 = Math.random()*20;
				//random throw to determin if it's adding or subtracting
				tmp = Math.random();
				(tmp>.5) ? gcoordx2 = gcoordx1+gcoordx2:gcoordx2 = gcoordx1-gcoordx2;
				gcoordy2 = Math.random()*20;
				tmp = Math.random();
				(tmp>.5) ? gcoordy2 = gcoordy1+gcoordy2:gcoordy2 = gcoordy1-gcoordy2;
				}
			}	
		// if it's "bubble", do this
		if (uiitt.selection.index==5)
			{
			if (i%2==1)
				{
				gcoordx1 = Math.random()*(docx);
				gcoordx1 = Math.ceil(gcoordx1);
				gcoordy1 = Math.random()*(docy);
				gcoordy1 = Math.ceil(gcoordy1);
				// define maximum size of the bubble based on document size and number of bubbles
				maxdist = docx/cycles*16;
				// generate co-ordinate to create a random number up to that size
				gcoordx2 = Math.random()*maxdist;
				gcoordy2 = gcoordx2;
				// add it to gcooordx1 to give a final co-ord for the bubble destination
				gcoordx2 = gcoordx1+gcoordx2;
				gcoordx2 = Math.round(gcoordx2);
				gcoordy2 = gcoordy1+gcoordy2;
				gcoordy2 = Math.round(gcoordy2);
				}
			else
				{
				gcoordx2=gcoordx2-Math.random()*(maxdist/4);
				gcoordx2 = Math.round(gcoordx2);
				gcoordy2=gcoordy2-Math.random()*(maxdist/4);
				gcoordy2 = Math.round(gcoordy2);
				}
			}
		//if it's "diamond" do this
		if (uiitt.selection.index==6)
			{
			if (i==1)
				{
				gcoordx1=Math.random()*3; 
				gcoordx1=Math.ceil(gcoordx1-1);
				gcoordx1=posix[gcoordx1];
				
				gcoordy1=Math.random()*3;
				gcoordy1=Math.ceil(gcoordy1-1);
				gcoordy1=posiy[gcoordy1];
				
				gcoordx2=Math.random()*3;
				gcoordx2=Math.ceil(gcoordx2-1);
				gcoordx2=posix[gcoordx2];
				
				gcoordy2=Math.random()*3;
				gcoordy2=Math.ceil(gcoordy2-1);
				gcoordy2=posiy[gcoordy2];
				}
			else
				{
				gcoordx1=gcoordx2;
				gcoordy1=gcoordy2;
				
				gcoordx2=Math.random()*3;
				gcoordx2=Math.ceil(gcoordx2-1);
				gcoordx2=posix[gcoordx2];
				
				gcoordy2=Math.random()*3
				gcoordy2=Math.ceil(gcoordy2-1);
				gcoordy2=posiy[gcoordy2];
				}
			}
		//if it's Spin or Twist, do this	
		if ((uiitt.selection.index==7)||(uiitt.selection.index==8))
			{
			var radius = docx/4;
			var xoff = docx/2;
			var yoff = docy/2;
			
			var cvari = Math.random()*(360/cycles); 
			cpos += (360/cycles)+cvari;
			
			gcoordx1 = radius* Math.cos( cpos*Math.PI/180 );
			gcoordy1 = radius* Math.sin( cpos*Math.PI/180 );
			gcoordx1 += xoff
			gcoordy1 += yoff
			
			gcoordx2 = Math.random()*((docx/3)+(101-cycles));
			tmp = Math.random();
			(tmp>.5) ? gcoordx2 = gcoordx1+gcoordx2:gcoordx2 = gcoordx1-gcoordx2;
			gcoordy2 = Math.random()*((docy/3)+(101-cycles));
			tmp = Math.random();
			(tmp>.5) ? gcoordy2 = gcoordy1+gcoordy2:gcoordy2 = gcoordy1-gcoordy2;
			}	
		softgrad();
		}
	}
		
/*
------------
FUNCTION: softgrad
------------
*/
function softgrad()
		{	
		//Do gradient fill
		var id23 = charIDToTypeID( "Grdn" );
		    var desc4 = new ActionDescriptor();
		    var id24 = charIDToTypeID( "From" );
		        var desc5 = new ActionDescriptor();
		        var id25 = charIDToTypeID( "Hrzn" );
		        var id26 = charIDToTypeID( "#Pxl" );
		        desc5.putUnitDouble( id25, id26, gcoordx1 ); //X source
		        var id27 = charIDToTypeID( "Vrtc" );
		        var id28 = charIDToTypeID( "#Pxl" );
		        desc5.putUnitDouble( id27, id28, gcoordy1 ); //Y source
		    var id29 = charIDToTypeID( "Pnt " );
		    desc4.putObject( id24, id29, desc5 );
		    var id30 = charIDToTypeID( "T   " );
		        var desc6 = new ActionDescriptor();
		        var id31 = charIDToTypeID( "Hrzn" );
		        var id32 = charIDToTypeID( "#Pxl" );
		        desc6.putUnitDouble( id31, id32, gcoordx2 ); //X destination
		        var id33 = charIDToTypeID( "Vrtc" );
		        var id34 = charIDToTypeID( "#Pxl" );
		        desc6.putUnitDouble( id33, id34, gcoordy2 ); //Y destination
		    var id35 = charIDToTypeID( "Pnt " );
		    desc4.putObject( id30, id35, desc6 );
		    var id36 = charIDToTypeID( "Md  " );
		    var id37 = charIDToTypeID( "BlnM" );
		    var id38 = charIDToTypeID( "Dfrn" );
		    desc4.putEnumerated( id36, id37, id38 );
		    var id39 = charIDToTypeID( "Type" );
		    var id40 = charIDToTypeID( "GrdT" );
		    var id41 = charIDToTypeID( filltypes );  //gradienr type "Rdl " "Angl" "Rflc" etc
		    desc4.putEnumerated( id39, id40, id41 );
		    var id42 = charIDToTypeID( "Dthr" );
		    desc4.putBoolean( id42, true );
		    var id43 = charIDToTypeID( "UsMs" );
		    desc4.putBoolean( id43, true );
		    var id44 = charIDToTypeID( "Grad" );
		        var desc7 = new ActionDescriptor();
		        var id45 = charIDToTypeID( "Nm  " );
		        desc7.putString( id45, "Black, White" ); //gradient fill type
		        var id46 = charIDToTypeID( "GrdF" );
		        var id47 = charIDToTypeID( "GrdF" );
		        var id48 = charIDToTypeID( "CstS" );
		        desc7.putEnumerated( id46, id47, id48 );
		        var id49 = charIDToTypeID( "Intr" );
		        desc7.putDouble( id49, 4096.000000 );
		        var id50 = charIDToTypeID( "Clrs" );
		            var list1 = new ActionList();
		                var desc8 = new ActionDescriptor();
		                var id51 = charIDToTypeID( "Clr " );
		                    var desc9 = new ActionDescriptor();
		                    var id52 = charIDToTypeID( "Cyn " );
		                    desc9.putDouble( id52, 75.020000 );
		                    var id53 = charIDToTypeID( "Mgnt" );
		                    desc9.putDouble( id53, 68.010000 );
		                    var id54 = charIDToTypeID( "Ylw " );
		                    desc9.putDouble( id54, 67.000000 );
		                    var id55 = charIDToTypeID( "Blck" );
		                    desc9.putDouble( id55, 90.190000 );
		                var id56 = charIDToTypeID( "CMYC" );
		                desc8.putObject( id51, id56, desc9 );
		                var id57 = charIDToTypeID( "Type" );
		                var id58 = charIDToTypeID( "Clry" );
		                var id59 = charIDToTypeID( "UsrS" );
		                desc8.putEnumerated( id57, id58, id59 );
		                var id60 = charIDToTypeID( "Lctn" );
		                desc8.putInteger( id60, 0 );
		                var id61 = charIDToTypeID( "Mdpn" );
		                desc8.putInteger( id61, 50 );
		            var id62 = charIDToTypeID( "Clrt" );
		            list1.putObject( id62, desc8 );
		                var desc10 = new ActionDescriptor();
		                var id63 = charIDToTypeID( "Clr " );
		                    var desc11 = new ActionDescriptor();
		                    var id64 = charIDToTypeID( "Cyn " );
		                    desc11.putDouble( id64, 0.000000 );
		                    var id65 = charIDToTypeID( "Mgnt" );
		                    desc11.putDouble( id65, 0.000000 );
		                    var id66 = charIDToTypeID( "Ylw " );
		                    desc11.putDouble( id66, 0.000000 );
		                    var id67 = charIDToTypeID( "Blck" );
		                    desc11.putDouble( id67, 0.000000 );
		                var id68 = charIDToTypeID( "CMYC" );
		                desc10.putObject( id63, id68, desc11 );
		                var id69 = charIDToTypeID( "Type" );
		                var id70 = charIDToTypeID( "Clry" );
		                var id71 = charIDToTypeID( "UsrS" );
		                desc10.putEnumerated( id69, id70, id71 );
		                var id72 = charIDToTypeID( "Lctn" );
		                desc10.putInteger( id72, 4096 );
		                var id73 = charIDToTypeID( "Mdpn" );
		                desc10.putInteger( id73, 50 );
		            var id74 = charIDToTypeID( "Clrt" );
		            list1.putObject( id74, desc10 );
		        desc7.putList( id50, list1 );
		        var id75 = charIDToTypeID( "Trns" );
		            var list2 = new ActionList();
		                var desc12 = new ActionDescriptor();
		                var id76 = charIDToTypeID( "Opct" );
		                var id77 = charIDToTypeID( "#Prc" );
		                desc12.putUnitDouble( id76, id77, 100.000000 );
		                var id78 = charIDToTypeID( "Lctn" );
		                desc12.putInteger( id78, 0 );
		                var id79 = charIDToTypeID( "Mdpn" );
		                desc12.putInteger( id79, 50 );
		            var id80 = charIDToTypeID( "TrnS" );
		            list2.putObject( id80, desc12 );
		                var desc13 = new ActionDescriptor();
		                var id81 = charIDToTypeID( "Opct" );
		                var id82 = charIDToTypeID( "#Prc" );
		                desc13.putUnitDouble( id81, id82, 100.000000 );
		                var id83 = charIDToTypeID( "Lctn" );
		                desc13.putInteger( id83, 4096 );
		                var id84 = charIDToTypeID( "Mdpn" );
		                desc13.putInteger( id84, 50 );
		            var id85 = charIDToTypeID( "TrnS" );
		            list2.putObject( id85, desc13 );
		        desc7.putList( id75, list2 );
		    var id86 = charIDToTypeID( "Grdn" );
		    desc4.putObject( id44, id86, desc7 );
		executeAction( id23, desc4, DialogModes.NO );
	}