// SCRIPT NAME: vSavePSDTGA
var VERSION=3.76;
// AUTHOR: James A. Taylor
// COPYRIGHT: James A. Taylor
// DESCRIPTION: Allows the saving of both a PSD and a related TGA file at the same time.

/* 	
See required prefs document on what each pref setting means
(if you don't have the document you probably shouldn't be in here)
*/

/*
FUNCTION LIST

main
checkprefs
newprefs
sortprefs
psdtoui
globaltoui
storeallinfo
storeglobalinfo
savepsdprefs
saveglobalprefs
setupui
setupsets
getsetname
savepsd
createtganame
tgamagic
savetga
alphamagic
createnewalpha
resizemagic
normalmapmagic
callcommand
mergevisible
layerselection
*/


//VARIABLES

//Preference file variables
var prefs,prefpath,preffile,keys,pversion
// External Preferences Storage Variables
var ecomress,ehexcol,etganame,etgapath,esavealp,epsdname,epsdpath,eimgx,eimgy,enmsave,enmdpth,eresmet,enmwrap,ecmdline,ecmdrun,elyrsets,enminv,esetmark,etgaopen;
// PSD Keyword Prefs Storage Variables
var khexcol,ktganame,ktgapath,ksavealp,kpsdname,kpsdpath,kimgx,kimgy,knmsave,knmdpth,kresmet,knmwrap,kcmdline,kcmdrun,klyrsets,knminv,ksetmark,ktgaopen;
// Internal Prefs Storage Variables
var ptganame,ptgapath,psavealp,ppsdname,ppsdpath,pimgx,pimgy,pnmsave,pnmdpth,presmet,pnmwrap,pcmdline,pcmdrun,plyrsets;
// File information variables
var pdispath,tganame,thetga,psdname,thepsd,psdsaved,tgasaved,changemenot=true,tganamestore;
// useful other variables
var tempstore,utgapath,utganame,setloop,j,alphaname,forcepsd,chanlength,thelayer,showhex;

#target photoshop
app.bringToFront();

//Get the whole thing moving
main();

//FUNCTIONS

/*
------------
FUNCTION: main
------------
*/
function main()
	{
	// check if there are actually any documents open on which to perform the action
	if (documents.length <= 0)
		{
		alert("Sorry, the script requires a document to save.","ERROR: Required parameters not met");
		return;
		}
		
	//store the default setting that may be affected
	var myunit = preferences.rulerUnits;

	//set the current preference to be pixels
	preferences.rulerUnits = Units.PIXELS;

	// set the variable "psdname" to the currently active documents name
	psdname=activeDocument.name;
	// set "thepsd" to point to the currently active document
	thepsd=activeDocument;
	
	//get the preferences
	checkprefs();
	//call the UI setup
	setupui();
		
	//now restore the default settings
	preferences.rulerUnits = myunit;
	}
	
/*
------------
FUNCTION: checkprefs
> Checks to see if the path and file for the preferences exists, and if they do loads them into a variable.
------------
*/
function checkprefs()
	{
	//create the defaults - this way if the user has an older version of the file 
	//they can still use the script with new options just using the defaults
	newprefs(); 
	//set prefpath to point to the users Application Data folder, or local OS equivalent
	prefpath=Folder.userData;
	//set it to point to the vPSScriptPrefs folder within the users Application Data Folder
	prefpath=new Folder(prefpath+"/vPSScriptPrefs");
	//set preffile to point to the preferences file within the prefpath 
	preffile=new File(prefpath+"/vsavepsdtga_prefs.jtp");
	//check if the path exists
	if (prefpath.exists) 
		{
		if (preffile.exists)
			{
			//open the preference file
			preffile.open("r");
			prefs=preffile.read()
			//split the resulting string into an array
			prefs=prefs.toString();
			//firstly, see if the existing preferences are current for version 3.5 or above (prefs version 2), use the defaults if they aren't.
			if (prefs.indexOf("pversion = 2")==-1) 
				{
				alert("Default preferences from versions of vSavePSDTGA prior to 3.5 are incompatible with this version.\nReseting to defaults, please recreate your default preferences manually.\n\nThis will not effect Preferences associated with PSD's","ERROR: Old Preferences Found");
				//set the prefs version back to two now we've avoided the eval
				pversion=2.0
				}
			else {eval(prefs)};
			preffile.close();
			}
		else {newprefs();}
		}
	//if the path didn't exist, create it
	else 
		{
		prefpath.create();
		newprefs();
		}
	//Now take that preference data and do something useful with it.
	sortprefs();
	}
	
/*
------------
FUNCTION: newprefs
creates a set of new default preferences for the script to access.
------------
*/
function newprefs()
	{
	pversion = 2
	etganame = "Untitled"; 	// Name of TGA if not same as PSD (string)
	etgapath = "NEW"; 	    // Path of TGA if not same as PSD (path) 
	esavealp = 0; 	// Alpha channel settings (number, 0 is Don't Save, 1 is Save Highest, 2 is Based on Sets)
	ehexcol  = "ff00ff" //hex color value for chroma key color (string)
	epsdname = true; 	// Use same name as PSD (true/false)
	epsdpath = true; 	// Use same path as PSD (true/false)
	eimgx	 = 0; 	// X size of TGA (number, where 0 is Same as PSD, 1 is 64 and 5 is 1024)
	eimgy 	 = 0; 	// Y size of TGA (number, where 0 is Same as PSD, 1 is 64 and 5 is 1024)
	enmsave  = 0; 	// Whether to generate a normal map for the image (number, 0 is Don't, 1 is do, 2 is Normalize)
	enmdpth  = 2.5; 	// The depth of the normal map (number - 0 to 500, default 2.5)
	eresmet  = 1; 	// Resample Method index, to use on resize (number 1 thru 5)
	enmwrap  = false; 	// whether to set the wrap flag on the normal map (True/False)
	ecmdline = ""; 	// The command line command to run via a batch file (parsed Text String)
	ecmdrun  = false; 	// Whether to run the Batch file after saving the TGA (True/False)
	elyrsets = 0; 	// Save layersets as seperate images, and how to save (number 0 thru 4)
	enminv   = false;	// Whether to invert the heightmap when creating a normalmap (true/false).
	esetmark = "~"; // The symbol used to mark layersets for batch export
	etgaopen = "false"; //whether to leave open the TGA after saving
	ecompress = "true"; //whether to  compress the TGA when saving
	}
	
/*
------------
FUNCTION: sortprefs

------------
*/
function sortprefs()
	{
	//get the filepath from the preferences and convert it into something useable
	//if the ptgapath is "NEW", copy a default folder object into ptgapath
	if (etgapath=="NEW")
		{
		etgapath=Folder.myDocuments;
		etgapath=new Folder(etgapath);
		}
	//otherwise just convert the string into a folder object
	//check it exists, and if not set the path to MyDocuments
	else
		{
		etgapath=new Folder(etgapath);
		var temp=etgapath.exists;
		if (temp==false){etgapath=Folder.myDocuments;}
		}
	
	//Get the keyword information from the PSD and do the same thing.
	//Note: The PSD slots may not exist - this just results in an UNDEFINED tag that we can deal with later.
	var keys=thepsd.info.keywords;
	ktganame = keys[0]; 	// Name of TGA if not same as PSD (string)
	ktgapath = keys[1]; 	// Path of TGA if not same as PSD (path) 
	ksavealp = keys[2]; 	// Save alpha channel default if one is present (true/false)
	kpsdname = keys[3]; 	// Use same name as PSD (true/false)
	kpsdpath = keys[4]; 	// Use same path as PSD (true/false)
	kimgx	 = keys[5]; 	// X size of TGA (number, where 0 is Same as PSD, 1 is 64 and 5 is 1024)
	kimgy 	 = keys[6]; 	// Y size of TGA (number, where 0 is Same as PSD, 1 is 64 and 5 is 1024)
	knmsave  = keys[7]; 	// Whether to generate a normal map for the image (true/false)
	knmdpth  = keys[8]; 	// The depth of the normal map (number - 0 to 500, default 100)
	kresmet  = keys[9]; 	// Resample Method index, to use on resize (number 1 thru 5)
	knmwrap  = keys[10]; 	// whether to set the wrap flag on the normal map (True/False)
	kcmdline = keys[11]; 	// The command line command to run via a batch file (parsed Text String)
	kcmdrun  = keys[12]; 	// Whether to run the Batch file after saving the TGA (True/False)
	klyrsets = keys[13]; 	// Save layersets as seperate images, and how to save (number 0 thru 4)
	knminv   = keys[14];	// Whether to invert the heightmap when creating a normalmap (true/false).
	ksetmark = keys[15];	// The symbol used to designate layer sets to export as images (string).
	khexcol  = keys[16];	//hex color value for chroma key color (string)
	
	//process the data from the general PSD keys
	//if key is undefined then place "~~" in the slot
	if (typeof ktganame == "undefined") ktganame="~~";
	if (typeof ktgapath == "undefined") ktgapath="~~";
	if (typeof kimgx == "undefined") kimgx="~~";
	if (typeof kimgy == "undefined") kimgy="~~";
	if (typeof knmdpth == "undefined") knmdpth="~~";
	if (typeof kresmet == "undefined") kresmet="~~";
	if (typeof kcmdline == "undefined") kcmdline="~~";
	if (typeof klyrsets == "undefined") klyrsets="~~";
	if (typeof ksetmark == "undefined") ksetmark="~~";
	if (typeof khexcol == "undefined") khexcol="~~";
	
	//Process the booleans from the PSD keys to make them real booleans
	//if key is undefined then place "~~" in the slot
	if ((typeof kpsdname !=  "undefined")&&(kpsdname != "~~"))
		{(kpsdname=="true") ? kpsdname=true:kpsdname=false;}
	else {kpsdname="~~";}
	if ((typeof kpsdpath !=  "undefined")&&(kpsdpath != "~~"))
		{(kpsdpath=="true") ? kpsdpath=true:kpsdpath=false;}
	else {kpsdpath="~~";}
	if ((typeof knmwrap !=  "undefined")&&(knmwrap != "~~"))
		{(knmwrap=="true") ? knmwrap=true:knmwrap=false;}
	else {knmwrap="~~";}
	if ((typeof kcmdrun !=  "undefined")&&(kcmdrun != "~~"))
		{(kcmdrun=="true") ? kcmdrun=true:kcmdrun=false;}
	else {kcmdrun="~~";}
	if ((typeof knminv !=  "undefined")&&(knminv != "~~"))
		{(knminv=="true") ? knminv=true:knminv=false;}
	else {knminv="~~";}
	
	// since the format of the preference data for the alpha channel info has changed from a boolean to an index 
	// we need to treat that slightly differently from the rest, both from the PSD and the Global Prefs.
	// This will create a valid index if one does not exist.
	// first the PSD
	if ((typeof ksavealp ==  "undefined")||(ksavealp == "~~"))
		{ksavealp="~~";}
	else 
		//If the value for ksavealp is defined as faux boolean then convert to a matching index, else do nothing
		{switch(ksavealp)
			{
			case "false" : ksavealp=0; break;
			case "true" : ksavealp=1; break;
			//if the case is neither "true" or "false" then just leave it as it is.
			default : ksavealp=ksavealp;
			}
		}
	//Now for the Globals (the only difference between the switches is that this will be a real boolean, not a string)
	switch(esavealp)
			{
			case false : esavealp=0; break;
			case true : esavealp=1; break;
			default : esavealp=esavealp;
			}
			
	// And we also need to do this for the Normal Map prefs (doh!) It's identical to the alpha ones.
	if ((typeof knmsave ==  "undefined")||(knmsave == "~~"))
		{knmsave="~~";}
	else 
		{switch(knmsave)
			{
			case "false" : knmsave=0; break;
			case "true" : knmsave=1; break;
			default : knmsave=knmsave;
			}
		}

	switch(enmsave)
			{
			case false : enmsave=0; break;
			case true : enmsave=1; break;
			default : enmsave=enmsave;
			}
			
	// Get the filepath from the PSD key data if there is any to get
	if (ktgapath != "~~")
		{
		//convert the string to a useable path
		ktgapath=new Folder(ktgapath);
		//check the path exists
		var temp=ktgapath.exists;
		//if it doesn't exist, replace the broken path with the "~~" string
		if (temp==false) 
			{
			alert ("The prefered TGA path stored in the PSD\nwas corrupt, renamed or removed.\nReseting path to default...","ERROR: Path not found");
			ktgapath=Folder.myDocuments;
			}
		}
	}

/*
------------
FUNCTION: psdtoui
------------
*/
function psdtoui()
	{
	// Check PSD for a boolean for the "Use PSD Name" checkbox
	if (kpsdname=="~~")
		{
		// if there was none use the value from the global prefs  
		uchksaveas.value=epsdname;
		// and set the "Associate with PSD" field for "Name" to be false
		upname.value=false;
		// set up a temporary storage for the name information
		utganame=etganame;
		}
	else
		{
		// if data was found use the value from the keyword
		uchksaveas.value=kpsdname;
		// and set the "Associate with PSD" field for "Name" to be true, since it must be associated
		upname.value=true;
		// set up a temporary storage for the name information
		utganame=ktganame
		}
	// check to see if that boolean (Use PSD Name) we assesed was true, and if it was, copy the generic info into the name field
	if (uchksaveas.value==true){unametext.text="Same as PSD..."}
	// otherwise copy the temporary storage variable from above into the name field
	else {unametext.text=utganame;}
		
	// now do basically the same thing for the path information, use utgapther instead of tempstore as we'll need access later 
	if (kpsdpath=="~~")
		{
		uchksaveto.value=epsdpath;
		uppath.value=false;
		utgapath=etgapath;
		}
	else
		{
		uchksaveto.value=kpsdpath;
		uppath.value=true;
		utgapath=ktgapath;
		}
	if (uchksaveto.value==true){upathtext.text="Same as PSD..."}
	else 
		{
		// Convert the extendscript format path into a system path before displaying it.
		tempstore=utgapath.fsName;
		upathtext.text=tempstore;
		}
		
	// alpha channel information	
	if (ksavealp=="~~")
		{
		uchkalpsav.selection=esavealp;
		showhex=esavealp;
		upalph.value=false;
		}
	else
		{
		uchkalpsav.selection=ksavealp;
		showhex=ksavealp;
		upalph.value=true;
		}
		
	if (khexcol=="~~")
		{
		ualphexcol.text=ehexcol;
		}
	else
		{
		ualphexcol.text=khexcol;
		}	
		
	// sets dropdown
	if (klyrsets=="~~")
		{
		usetsave.selection=elyrsets;
		usetmark.text=esetmark;
		upsets.value=false;
		}
	else
		{
		usetsave.selection=klyrsets;
		usetmark.text=ksetmark;
		upsets.value=true;
		}
		
	// resize fields
	// see if the information is coming from the PSD or the Global prefs
	// since if one value is stored there they all must have some data, we only need to check one of those values initially.
	// later check each one individually in case some of the options are from preferences a previous version didn't have,
	// or the data in the PSD keywords became weirdly corrupted. 
	if (kimgx=="~~")
		{
		// if it's coming from the global prefs, copy the global resize values into the relevent UI components
		usizex.selection=eimgx;
		usizey.selection=eimgy;
		uscaletype.selection=eresmet;
		upsize.value=false;
		}
	else
		{
		(kimgx=="~~") ? usizex.selection=0:usizex.selection=kimgx;
		(kimgy=="~~") ? usizey.selection=0:usizey.selection=kimgy;
		(kresmet=="~~") ? kresmet.selection=0:uscaletype.selection=kresmet;
		upsize.value=true;		
		}
		
	// Normal Map Fields
	// much the same as the resize stuff.
	if (knmsave=="~~")
		{
		unormalmap.selection=enmsave;
		unormalwrap.value=enmwrap;
		unormalinv.value=enminv;
		unormdepth.value=enmdpth;
		unorminput.text=enmdpth;
		upnorm.value=false;
		}
	else
		{
		(knmsave=="~~") ? unormalmap.selection=0:unormalmap.selection=knmsave;
		(knmwrap=="~~") ? unormalwrap.value=false:unormalwrap.value=knmwrap;
		(knminv=="~~") ? unormalinv.value=false:unormalinv.value=knminv;
		(knmdpth=="~~") ? unormdepth.value=2.5:unormdepth.value=knmdpth;
		unorminput.text=unormdepth.value;
		upnorm.value=true;
		}
	
	// CMD fields - same stuff again
	if (kcmdrun=="~~")
		{
		ucmdrun.value=ecmdrun;
		ucmdtext.text=ecmdline;
		upcmd.value=false;
		}
	else
		{
		(kcmdrun=="~~") ? ucmdrun.value=false:ucmdrun.value=kcmdrun;
		(kcmdline=="~~") ? ucmdtext.text="":ucmdtext.text=kcmdline;
		upcmd.value=true;
		}
	//since the misc info is only saved in the Global prefs we don't have to check it against info stored in the PSD
	uchkopen.value=etgaopen;
	ucompres.value=ecompress;
	}
	
/*
------------
FUNCTION: globaltoui
does the same as PSD to UI, but only takes the info from the Global settings
------------
*/
function globaltoui()
	{
	uchksaveas.value=epsdname;
	utganame=etganame;
	if (uchksaveas.value==true){unametext.text="Same as PSD..."}
	else {unametext.text=utganame;}
	uchksaveto.value=epsdpath;
	utgapath=etgapath;
	if (uchksaveto.value==true){upathtext.text="Same as PSD..."}
	else 
		{
		tempstore=utgapath.fsName;
		upathtext.text=tempstore;
		}
	uchkalpsav.selection=esavealp;
	ualphexcol.text=ehexcol;
	usetsave.selection=elyrsets;
	usizex.selection=eimgx;
	usizey.selection=eimgy;
	uscaletype.selection=eresmet;
	unormalmap.selection=enmsave;
	unormalwrap.value=enmwrap;
	unormalinv.value=enminv;
	unormdepth.value=enmdpth;
	unorminput.text=enmdpth;
	ucmdrun.value=ecmdrun;
	ucmdtext.text=ecmdline;
	usetmark.text=esetmark;
	uchkopen.value=etgaopen;
	ucompres.value=ecompress;
	}
	
/*
------------
FUNCTION: storeallinfo
Stores all the information into the relevant variables based on the Association with PSD selection 
------------
*/		
function storeallinfo()
	{
	if (upname.value==true)
		{
		ktganame=utganame;
		kpsdname=uchksaveas.value;
		}
	else
		{
		ktganame="~~";
		kpsdname="~~";
		etganame=utganame;
		epsdname=uchksaveas.value;
		}

	if (uppath.value==true)
		{
		ktgapath=utgapath;
		kpsdpath=uchksaveto.value;
		}
	else
		{
		ktgapath="~~";
		kpsdpath="~~";
		etgapath=utgapath;
		epsdpath=uchksaveto.value;
		}
	
	if (upsize.value==true)
		{
		kimgx=usizex.selection.index;
		kimgy=usizey.selection.index;
		kresmet=uscaletype.selection.index;
		}
	else
		{
		kimgx="~~";
		kimgy="~~";
		kresmet="~~";
		eimgx=usizex.selection.index;
		eimgy=usizey.selection.index;
		eresmet=uscaletype.selection.index;
		}
		
	if (upalph.value==true)
		{
		ksavealp=uchkalpsav.selection.index;
		khexcol=ualphexcol.text;
		}
	else
		{
		ksavealp="~~";
		esavealp=uchkalpsav.selection.index;
		ehexcol=ualphexcol.text;
		}
				
	if (upnorm.value==true)
		{
		knmsave=unormalmap.selection.index;
		knmdpth=unormdepth.value;
		knmwrap=unormalwrap.value;
		knminv=unormalinv.value;
		}
	else
		{
		knmsave="~~";
		knmdpth="~~";
		knmwrap="~~";
		knminv="~~";
		enmsave=unormalmap.selection.index;
		enmdpth=unormdepth.value;
		enmwrap=unormalwrap.value;
		enminv=unormalinv.value;
		}
	
	if (upcmd.value==true)
		{
		kcmdrun=ucmdrun.value;
		kcmdline=ucmdtext.text;
		}
	else
		{
		kcmdrun="~~";
		kcmdline="~~";
		ecmdrun=ucmdrun.value;
		ecmdline=ucmdtext.text;
		}
		
	if (upsets.value==true)
		{
		klyrsets=usetsave.selection.index;
		ksetmark=usetmark.text
		}
	else
		{
		klyrsets="~~";
		ksetmark="~~";
		elyrsets=usetsave.selection.index;
		esetmark=usetmark.text
		}
		
	(uchkopen.value==true) ? etgaopen=true:etgaopen=false;
	(ucompres.value==true) ? ecompress=true:ecompress=false;
	}
	
/*
------------
FUNCTION: storeglobalinfo
stores all the visible information into the global preferences variables
------------
*/		
function storeglobalinfo()
	{
	etganame=utganame;
	epsdname=uchksaveas.value;

	etgapath=utgapath;
	epsdpath=uchksaveto.value;

	eimgx=usizex.selection.index;
	eimgy=usizey.selection.index;
	eresmet=uscaletype.selection.index;
	
	esavealp=uchkalpsav.selection.index;
	ehexcol=ualphexcol.text;
			
	enmsave=unormalmap.selection.index;
	enmdpth=unormdepth.value;
	enmwrap=unormalwrap.value;
	enminv=unormalinv.value;

	ecmdrun=ucmdrun.value;
	ecmdline=ucmdtext.text;
	
	elyrsets=usetsave.selection.index;
	esetmark=usetmark.text;
	
	etgaopen=uchkopen.value;
	ecompress=ucompres.value;
	}
	
/*
------------
FUNCTION: savepsdprefs
stores PSD preferences to the PSD
------------
*/	
function savepsdprefs()
	{	
	if ((ktganame=="~~")&&(ktgapath=="~~")&&(ksavealp=="~~")&&(kpsdname=="~~")&&(kpsdpath=="~~")&&(kimgx=="~~")&&(kimgy=="~~")&&(knmsave=="~~")&&(knmdpth=="~~")&&(kresmet=="~~")&&(knmwrap=="~~")&&(kcmdline=="~~")&&(kcmdrun=="~~")&&(klyrsets=="~~")&&(knminv=="~~")&&(ksetmark=="~~")&&(khexcol=="~~"))
		{
		keywordarray=new Array();
		}
	else
		{
		//convert ktgapath to a string
		ktgapath=ktgapath.toString();
		keywordarray=new Array(ktganame,ktgapath,ksavealp,kpsdname,kpsdpath,kimgx,kimgy,knmsave,knmdpth,kresmet,knmwrap,kcmdline,kcmdrun,klyrsets,knminv,ksetmark,khexcol);
		}
	
	//write the array to the PSD
	thepsd.info.keywords=keywordarray;
	}
	
/*
------------
FUNCTION: saveglobalprefs
saves Global Prefs
------------
*/		
function saveglobalprefs()
	{
	//convert the CMD line text into a safe format for storage (horrid piece of code, but I don't know a better way - most of it's comments anyway):
	
	//setup array containing "bad" characters we need to adjust:  "  '  \
	var badsymbl = new Array("\\","\"","\'");
	// setup a loop, copy ecmdline into fullstring (to avoid corruption) and set up the variable stringend
	var ii=0;
	var fullstring=ecmdline;
	var stringend;
	//do the following until all the symbols have ben searched for and adjusted
	while (ii<3)
		{
		// set up i as 0 to control the second loop, which will l
		// copy the command line into stringend so we have a string that can recursively shorten and then clear fullstring so we have a clean container
		i=0;
		stringend=fullstring;
		fullstring="";
		//loop until all instances of the current character are altered. 
		while (i<5)
			{
			// search stringend for the required symbol to add a \ to
			var temp=stringend.indexOf(badsymbl[ii]);
			// if it was found, do all this:
			if (temp!=-1)
				{
				// copy the string up to the symbol into stringstart
				var stringstart = stringend.substring(0,temp);
				// copy the end of the string, minus the required symbol, into stringend.
				stringend = stringend.substring(temp+1,stringend.length+1);
				// now put them back together with the symbol prefixed by \, leaving off stringend as that's added on the next loop. 
				// fullstring - starting blank - will slowly fill up as stringend gets smaller
				fullstring=fullstring+stringstart+"\\"+badsymbl[ii];//+stringend;
				}
			//otherwise set i to exit the loop, make sure fullstring is the full revised commandline and clear stringend ready  for the next main loop
			else
				{
				i=5;
				fullstring=fullstring+stringend;
				stringend="";
				}
			i=i+1
			}
		ii++
		}
	
	//copy the pref variables into saveprefs - add the numeric variables, then the booleans, then the strings
	//note we're using the variable fullstring instead of ecmdline, but when reloaded that information will be processed as ecmdline.
	var saveprefs='pversion = 2;\neimgx = ' +eimgx+ ';\neimgy = ' +eimgy+ ';\nenmdpth = ' +enmdpth+ ';\neresmet = ' +eresmet+ ';\nelyrsets = ' +elyrsets+ ';';
	var saveprefs=saveprefs+'\nesavealp = ' +esavealp+ ';\nepsdname = ' +epsdname+ ';\nepsdpath = ' +epsdpath+ ';\nenmsave = ' +enmsave+ ';\nenmwrap = ' +enmwrap+ ';\necmdrun = ' +ecmdrun+ ';\nenminv = ' +enminv+ ';\netgaopen = ' +etgaopen+ ';\necompress = ' +ecompress+ ';';
	var saveprefs=saveprefs+'\netganame = \"' +etganame+ '\";\netgapath = \"' +etgapath+ '\";\necmdline = \"' +fullstring+ '\";\nesetmark =\"' +esetmark+ '\";\nehexcol =\"' +ehexcol+ '\";';
	//then open the file, destroying the contents,
	preffile.open("w");
	//then write the prefs to the file and close it.
	preffile.write(saveprefs);
	preffile.close();
	}

/*
------------
FUNCTION: setupui
------------
*/	
function setupui()
	{
	setupmainwindow = 
		"dialog { text: 'vSavePSDTGA', orientation: 'column' \
			, mainpanel: Group {orientation: 'column' \
				, row1: Group {orientation: 'row' \
					, preftype: Button {text:'PSD', helpTip:'Cycles between editing a combination of PSD specific and Global Preferences, and editing only Global preferences.'} \
					, psdprefs: Panel { text: 'Associate With PSD:', orientation:'row', helpTip:'Checked preferences are saved with the PSD file.'\
						, pname: Checkbox {text:'Name', value:false, helpTip:'Output name preferences are saved with the PSD file.'} \
						, ppath: Checkbox {text:'Path', value:false, helpTip:'Output path preferences are saved with the PSD file.'} \
						, psize: Checkbox {text:'Size', value:false, helpTip:'Output size preferences are saved with the PSD file.'} \
						, palph: Checkbox {text:'Alpha', value:false, helpTip:'Output Alpha Channel preferences are saved with the PSD file.'} \
						, pnorm: Checkbox {text:'Normal', value:false, helpTip:'Normal Map preferences are saved with the PSD file.'} \
						, pcmd:	Checkbox {text:'CMD', value:false, helpTip:'Command Line preferences are saved with the PSD file.'} \
						, psets: Checkbox {text:'Sets', value:false, helpTip:'Layerset save type preferences are saved with the PSD file.'} \
					}\
				}\
				, div1: Group {orientation: 'row', alignment:'center' \
					, divtxt1: StaticText {text:'Basic'} \
					, divider1: Panel {orientation: 'row', alignment:'center', preferredSize:[465,2] \
					, properties: {borderStyle: 'black'} \
					}\
				}\
				, row2: Group {orientation: 'row', alignment: 'center' \
					, ezcol3: Group {orientation: 'column',alignment: 'right' \
						, setrow: Group {orientation: 'row', alignment: 'right' \
							, settext: StaticText {text:'Sets:', alignment:'center'} \
							, setsave: DropDownList {helpTip:'Selects the manner in which root marked Layersets are treated.', properties:{items:['Ignore marked layer sets','Save highest, append _initial','Save highest, same name','Save highest, append name','Save all, append _initial','Save all, same name','Save all, append name']} } \
						}\
						, setsymgrp: Group {orientation: 'row', alignment:'right' \
							, setsymhlp: StaticText {text:'Root layer set marker: '} \
							, setsym: EditText {helpTip:'Defines the symbol to use as a layerset marker. The default symbol is the ~ (Tilde).' , text:'~', preferredSize:[20,20]} \
						}\
					}\
					, divcol: Panel {orientation: 'column', alignment:'center', preferredSize:[2,50], properties: {borderStyle: 'sunken'} \
					}\
					, ezcol1: Group {orientation: 'column', alignment: 'left' \
						, alphrow: Group {orientation: 'row', alignment: 'right' \
							, alptext1: StaticText {text:'Alpha Channel:'}\
							, chkalpsav: DropDownList {helpTip:'Save Alpha in this way (if relevant alpha type exists)', alignment:'left', properties:{items:['Do Not Save Alpha','Highest Alpha Only','Associated With Marked Sets','Image Transparency (8bit)','Image Transparency (2bit)','Chroma Key Only']} }\
						}\
						, alpstk: Group {orientation:'Stack' \
							, alphtext2: StaticText {text:'Image contains no Alpha', alignment:'left'} \
							, alpkeyrow: Group {Orientation:'row', alignment:'left' \
								, alpkeytxt: Button {text: 'Key Color (hex)' } \
								, alpkeycol: EditText {text:'ff00ff',preferredSize:[60,20], helpTip:'The Hex Value of the colour intended to replace the image transparency.'} \
								, alpkey: Panel {preferredSize:[18,18], properties:{borderStyle:'sunken' }} \
								}\
							}\
					}\
				}\
				, divider2: Panel {orientation: 'row', alignment:'center', preferredSize:[500,2] \
				, properties: {borderStyle: 'sunken'} \
				}\
				, resizegroup: Group {orientation: 'row', alignment:'center' \
					, restext: StaticText {text:'Resize TGA to:'} \
					, sizex: DropDownList { helpTip:'The width at which to save the output TGA.', properties:{items:['No Change','64','128','256','512','1024','2048']} } \
					, restextx: StaticText {text:'W x H'} \
					, sizey: DropDownList { helpTip:'The height at which to save the output TGA.',properties:{items:['No Change','64','128','256','512','1024','2048']} } \
					, restextr: StaticText {text:'Resample:'} \
					, scaletype: DropDownList {helpTip:'The resample method used to save the output TGA.',properties:{items:['None','Bicubic','Bicubic (Sharp)','Bicubic (Soft)','Bilinear','Nearest Neighbour']} } \
				}\
				, divider3: Panel {orientation: 'row', alignment:'center', preferredSize:[500,2] \
				, properties: {borderStyle: 'sunken'} \
				}\
				, psdchkgroup: Group {orientation: 'row',alignment:'left'\
					, chksavas: Checkbox {text:'Use PSD Name', value:false, alignment:'left', helpTip:'This will save the output TGA using the same name as the source PSD.'} \
					, chksaveto: Checkbox {text:'Use PSD Path', value:false, alignment:'left', helpTip:'This will save the output TGA to the same path as the source PSD.'} \
				}\
				, namegroup: Group {orientation: 'row', alignment:'left' \
					, ntext: StaticText {text:'Name:'}\
					, nametext: EditText {text:'...', alignment:'left', preferredSize:[457,20], helpTip:'The base name for the output TGA if not the same as the PSD. Does not require appending of \".tga\" as this will be added automatically',} \
				}\
				, pathgroup: Group {orientation: 'row', alignment:'left' \
					, ptext: StaticText {text:'Path:  '}\
					, pathgrp: Group {orientation: 'stack', alignment: 'left' \
						, pathtext: StaticText {helpTip:'The location to which the output TGA will be saved if different from the source PSD.', text:'...', alignment:'center', preferredSize:[410,22] } \
						, pathpan: Panel {text:'', alignment:'left', preferredSize:[415,25]} \
						, properties: {borderStyle: 'black'} \
					}\
					, selpath: Button {helpTip:'Click to select the output folder.',text:'...', preferredSize:[32,24]} \
				}\
				, div4: Group {orientation: 'row', alignment:'center' \
					, divtxt4: StaticText {text:'Normals'} \
					, divider4: Panel {orientation: 'row', alignment:'center', preferredSize:[451,2] \
					, properties: {borderStyle: 'black'} \
					}\
				}\
				, normalgroup: Group {orientation:'column',alignment:'center', preferredSize:[395,20] \
					, normalgroup1: Group {orientation:'row', alignment:'left' \
						, normtext: StaticText {text: 'Run Filter using:'} \
						, normalmap: DropDownList {helpTip:'Do Not Run Filter: will not run the filter (sets marker will override) Auto Select Filter Type: Tries to calculate whether to convert a height map to a normal map, or to Normalize an existing normal. The remaining options force this distinction.  Does not affect the source PSD.' , properties:{items:['Do not run filter','Auto Select Filter Type', 'Convert Bumps to Normals','Normalize Existing Normals']} } \
						, normalwrap: Checkbox {text: 'Wrap', value:false, helpTip:'Alters the way the normal map generation handles pixels at the edge of the image.'} \
						, normalinv: Checkbox {text: 'Invert Normals', value:false, helpTip:'Reverses the way the normal map generation handles lights and darks from the input PSD.',} \
					}\
					, normalgroup2: Group {orientation: 'row', alignment:'left' \
						, normtext:	StaticText {text:'Depth:'} \
						, normdepth: Slider {value:2.2,minvalue:0,maxvalue:100,preferredSize:[250,21],alignment:'center', helpTip:'Reflects the depth of the normal map as based on the difference between White and Black in the source PSD.  Minimum is 0, Maximum is 100.'} \
						, norminput: EditText {text:'2.2',preferredSize:[30,20], helpTip:'Reflects the depth of the normal map as based on the difference between White and Black in the source PSD.  Minimum is 0, Maximum is 100.'} \
						, normreset: Button { text:'Reset Depth', helpTip:'Resets the output Normal Map depth to the default of 2.5.'} \
					}\
				}\
				, div5: Group {orientation: 'row', alignment:'center' \
					, divtxt5: StaticText {text:'Command Line'} \
					, divider5: Panel {orientation: 'row', alignment:'center', preferredSize:[420,2] \
					, properties: {borderStyle: 'black'} \
					}\
				}\
				, cmdgroup: Group {orientation:'column', alignment: 'left' \
					, cmdrun: Checkbox {text:'Run CMD on save', value:false, alignment: 'left', helpTip:'This will open a batch file to run the Command Line options listed below after the TGA is saved.'} \
					, cmdrow1: Group {orientation: 'row', alignment:'left' \
						, ctext: StaticText {text:'CMD:'}\
						, cmdtext: EditText {text:'...', alignment:'left', preferredSize:[460,20], helpTip:'The Command Line options to run after the TGA is saved'} \
					}\
					, cmdrow2: Group {orientation:'row', alignment:'center' \
						, cmdtxt: StaticText {text:'Append to CMD:'} \
						, cmdnam: Button {text: 'Name', helpTip:'Adds the name of the TGA to the end of the Command Line string.  This is shown as <FILENAME> but will be converted to the name listed in the Name field when run.'} \
						, cmdpat: Button {text: 'Path', helpTip:'Adds the path of the TGA to the end of the Command Line string.  This is shown as <FILEPATH> but will be converted to the path listed in the Path field when run.'} \
						, droptxt: StaticText {text:'Set CMD to:'} \
						, cmddrp: Button {text: 'Droplet...', helpTip:'Sets the CMD line to run a selected Adobe Droplet on the TGA after saving. This replaces any existing CMD information.'} \
					}\
				}\
				, div6: Group {orientation: 'row', alignment:'center' \
					, divtxt6: StaticText {text:'Misc'} \
					, divider6: Panel {orientation: 'row', alignment:'center', preferredSize:[469,2] \
					, properties: {borderStyle: 'black'} \
					}\
				}\
				, miscgroup: Group { orientation:'row', alignment:'center', alignclildren:'center' \
					, compress: Checkbox {text: 'Compress TGA (RLE)', value:true, alignment: 'center', helpTip:'Sets whether to use RLE compression when saving the TGA. The state of the checkbox is saved only with the global preferences.'} \
					, chkopen: Checkbox {text:'Leave TGA open after saving', value:false, alignment: 'center', helpTip:'When checked this option will leave the TGA file open in Photoshop subsiquent to saving it.  The state of the checkbox is saved only with the global preferences.'} \
					}\
				, divider7: Panel {orientation: 'row', alignment:'center', preferredSize:[500,2] \
				, properties: {borderStyle: 'sunken'} \
				}\
			}\
			, btngroup: Group { orientation:'row', alignment:'center', alignclildren:'center' \
				, savebtn: Button { text:'Save Prefs', helpTip:'Save the preferences shown above. The location of the preferences to be saved is dependent on the checks in the \"Associate With PSD\" panel.'} \
				, buttonsplit1: StaticText { text:'', preferredSize:[50,20]} \
				, usebtn: Button { text:'Save Images', helpTip:'Save both the PSD source (as is) and the TGA image based on the values above. Any unsaved preferences will be lost.', properties:{name:'OK'} } \
				, tgabtn: Button { text:'Save TGA Only', helpTip:'Save a TGA from the PSD source based on the values above.  Any unsaved preferences will be lost.' } \
				, buttonsplit2: StaticText { text:'', preferredSize:[50,20]} \
				, clbtn: Button { text:'Cancel', helpTip:'Exit the script and take no action at all. This will not revert saved preferences and any unsaved preferences will be lost.'} \
			}\
		}";
		
	//initialise the window
	mainwindow = new Window(setupmainwindow);
	//copy the version number into the window title
	mainwindow.text=mainwindow.text+" v"+VERSION;
	
	//set up variables for each useable UI component:
	upreftype	= mainwindow.mainpanel.row1.preftype;
	upsdprefs	= mainwindow.mainpanel.row1.psdprefs;
	upname		= mainwindow.mainpanel.row1.psdprefs.pname;
	uppath		= mainwindow.mainpanel.row1.psdprefs.ppath;
	upsize		= mainwindow.mainpanel.row1.psdprefs.psize;
	upalph		= mainwindow.mainpanel.row1.psdprefs.palph;
	upnorm		= mainwindow.mainpanel.row1.psdprefs.pnorm;
	upcmd		= mainwindow.mainpanel.row1.psdprefs.pcmd;
	upsets		= mainwindow.mainpanel.row1.psdprefs.psets;
	
	uchksaveto	= mainwindow.mainpanel.psdchkgroup.chksaveto;
	uchkalpsav	= mainwindow.mainpanel.row2.ezcol1.alphrow.chkalpsav;
	uchkalstat	= mainwindow.mainpanel.row2.ezcol1.alpstk.alphtext2;
	ualphextxt	= mainwindow.mainpanel.row2.ezcol1.alpstk.alpkeyrow.alpkeytxt;
	ualphexcol	= mainwindow.mainpanel.row2.ezcol1.alpstk.alpkeyrow.alpkeycol;
	ualpkey		= mainwindow.mainpanel.row2.ezcol1.alpstk.alpkeyrow.alpkey;
	uchksaveas	= mainwindow.mainpanel.psdchkgroup.chksavas;
	usetsave	= mainwindow.mainpanel.row2.ezcol3.setrow.setsave;
	usetmark	= mainwindow.mainpanel.row2.ezcol3.setsymgrp.setsym;
	
	usizex		= mainwindow.mainpanel.resizegroup.sizex;
	usizey		= mainwindow.mainpanel.resizegroup.sizey;
	uscaletype	= mainwindow.mainpanel.resizegroup.scaletype;
	
	unametext	= mainwindow.mainpanel.namegroup.nametext;
	upathtext	= mainwindow.mainpanel.pathgroup.pathgrp.pathtext;
	uselpath	= mainwindow.mainpanel.pathgroup.selpath;
	
	unormalmap	= mainwindow.mainpanel.normalgroup.normalgroup1.normalmap;
	unormalwrap	= mainwindow.mainpanel.normalgroup.normalgroup1.normalwrap;
	unormalinv	= mainwindow.mainpanel.normalgroup.normalgroup1.normalinv;
	unormdepth	= mainwindow.mainpanel.normalgroup.normalgroup2.normdepth;
	unorminput	= mainwindow.mainpanel.normalgroup.normalgroup2.norminput;
	unormreset	= mainwindow.mainpanel.normalgroup.normalgroup2.normreset;
	
	ucmdrun		= mainwindow.mainpanel.cmdgroup.cmdrun;
	ucmdtext	= mainwindow.mainpanel.cmdgroup.cmdrow1.cmdtext;
	ucmdnam		= mainwindow.mainpanel.cmdgroup.cmdrow2.cmdnam;
	ucmdpat		= mainwindow.mainpanel.cmdgroup.cmdrow2.cmdpat;
	ucmddrp		= mainwindow.mainpanel.cmdgroup.cmdrow2.cmddrp;
	
	uchkopen	= mainwindow.mainpanel.miscgroup.chkopen;
	ucompres	= mainwindow.mainpanel.miscgroup.compress;
	
	usavebtn	= mainwindow.btngroup.savebtn;
	uusebtn		= mainwindow.btngroup.usebtn;
	utgabtn		= mainwindow.btngroup.tgabtn;
	uclbtn		= mainwindow.btngroup.clbtn;
	
	//check how to present the alphachannel dropdown
	var psdalphachannel=thepsd.channels.length;
	if (psdalphachannel>4) uchkalstat.text="Image has "+(psdalphachannel-3)+" Alpha channels";
	if (psdalphachannel==4) uchkalstat.text="Image has 1 Alpha channel";
	
	// get the preferences information and place it into the various components
	// use psdtoui to split the prefs from the PSD and the Global file.
	psdtoui();
	
	//get the color to place in the color box
	colorchange();
	//store the initial color value
	var ghexcol=ualphexcol.text;
	
		//should the interface hide the hex value, or the alpha stats?
	if (showhex>2)
		{
		uchkalstat.visible = false;
		}
	else
		{
		//hide the hex col value
		ualphextxt.visible = false;
		ualphexcol.visible = false;
		ualpkey.visible = false;
		}
	
	// what mark should be used to define the layersets?
	usetmark.text=usetmark.text.substring(0,1);
	
	// now set up the component functionality.
	
	// set the alpha channel functionality
	uchkalpsav.onChange=function()
		{
		//should the interface hide the hex value, or the alpha stats?
		if (uchkalpsav.selection.index>2)
			{
			uchkalstat.visible = false;
			ualphextxt.visible = true;
			ualphexcol.visible = true;
			ualpkey.visible = true;
			}
		else
			{
			uchkalstat.visible = true;
			//hide the hex col value
			ualphextxt.visible = false;
			ualphexcol.visible = false;
			ualpkey.visible = false;
			}
		}
		
	//set the hexcolor validity check
	ualphexcol.onChange=function()
		{
		var hexcol=ualphexcol.text;
		try
			{
			var backcol=new SolidColor();
			backcol.rgb.hexValue=hexcol;
			ehexcol=ualphexcol.text;
			ghexcol=ualphexcol.text
			}
		catch(err)
			{
			alert ("Invalid hexadecimal color value. Reverting to last valid setting.\n\nValid settings are 6 characters, each between 0 and F in base 16.");
			ualphexcol.text=ghexcol;
			}
		colorchange();
		}
	
	//set the functionality of the hex color box
	ualphextxt.onClick=function()
		{
		var hexcol="0x"+ualphexcol.text;
		hexcol=$.colorPicker(hexcol);
		if (hexcol==0000-1) return;
		hexcol=validhexcol(hexcol);
		//alert(hexcol);
		ualphexcol.text = hexcol;
		colorchange();
		ghexcol=hexcol;
		}
		
	//set folder select button functionality
	uselpath.onClick=function()
		{
		//open the folder dialogue, using the predefined path as the default
		tempstore=utgapath.selectDlg("Select Folder for TGA...");
		//if the user clicked cancel then exit without storing the result
		if(tempstore==null) return;
		//store the script format path in utgapath, and then convert temp path to a file system one and copy it into the UI
		utgapath=tempstore; 
		tempstore=tempstore.fsName;
		upathtext.text=tempstore;
		//see if the "same as psd" checkbox is checked - if it is, uncheck it
		if (uchksaveto.value==true) {uchksaveto.value=false;}
		}
	
	//set name field functionality
	unametext.onChanging=function()
		{
		utganame=unametext.text
		uchksaveas.value=false;
		}
		
	//set mark field functionality
	usetmark.onChanging=function()
		{
		usetmark.text=usetmark.text.substring(0,1);
		}
	
	//set Use PSD Path functionality
	uchksaveto.onClick=function()
		{
		if (uchksaveto.value==false) 
			{
			tempstore=utgapath.fsName;
			upathtext.text=tempstore;
			}
		else 
			{
			upathtext.text="Same as PSD...";
			}
		}
	
	//set Use PSD Name functionality
	uchksaveas.onClick=function()
		{
		if (uchksaveas.value==false) 
			{
			unametext.text=utganame;
			}
		else 
			{
			unametext.text="Same as PSD...";
			}
		}
		
	//set normdepth functionality
	unormdepth.onChanging=function()
		{
		//slider functionality
		// convert the input number into a useable length
		var commu = unormdepth.value;
		commu = commu * 10;
		commu = Math.round(commu);
		commu /= 10;
		// copy corrected number into box and slider
		unormdepth.value=commu;
		unorminput.text=commu;
		}
		
	//set unormreset functionality
	unormreset.onClick=function()
		{
		// copy the default number into box and slider
		unormdepth.value=2.5;
		unorminput.text=2.5;
		}
		
	//set norminput functionality
	unorminput.onChange=function()
		{
		//textbox functionality
		//set up temporary var and copy text input into it
		var tnum=unorminput.text;
		//convert text into a floating point number
		tnum=parseFloat(tnum);
		// verify the input actually was a number
		if (isNaN(tnum)==true)
			{
			alert("Invalid Value!\nResetting to last valid number.","ERROR: Numeric Range Required");
			unorminput.text=unormdepth.value;
			}
		else
			{
			// clip the number to fit between the max and min amounts
			if (tnum < 0) tnum = 0;
			if (tnum > 100) tnum = 100;
			// then convert the number into a useable length
			var commu = tnum;
			commu = commu * 10;
			commu = Math.round(commu);
			commu /= 10;
			tnum = commu;
			//copy the converted value back into the box and also into the associated slider bar.
			unormdepth.value=tnum;
			unorminput.text=tnum;
			}
		}
	
	//Set command append button functionalities
	ucmdnam.onClick=function()
		{
		ucmdtext.text=ucmdtext.text+"<FILENAME>";
		}
			
	ucmdpat.onClick=function()
		{
		ucmdtext.text=ucmdtext.text+"<FILEPATH>";
		}
	
	//Set Droplet... button functionality
	ucmddrp.onClick=function()
		{
		//set dropit to be a folder object pointing to the desktop
		var dropit=File("Desktop");
		//open an open dialogue in that folder - dropit becomes the selected file
		dropit=dropit.openDlg("Select Droplet to Run:","Droplets:*.exe");
		//if the user actually did select a file do this.
		if (dropit!=null)
			{
			// convert the internal path to a system path
			dropit=dropit.fsName;
			//copy all the data into the command line:
			ucmdtext.text="\""+dropit+"\" \"<FILEPATH><FILENAME>\"";
			}
		}
	
	//Set PSD/GLOBAL button functionality
	upreftype.onClick=function()
		{
		if (upreftype.text=="PSD")
			{
			storeallinfo();
			globaltoui();
			upsdprefs.visible=false;
			upreftype.text="GLOBAL";
			//get the color to place in the color box
			colorchange();
			//store the initial color value
			ghexcol=ualphexcol.text;
			}
		else
			{
			storeglobalinfo();
			psdtoui();
			upsdprefs.visible=true;
			upreftype.text="PSD";
			//get the color to place in the color box
			colorchange();
			//store the initial color value
			ghexcol=ualphexcol.text;
			}
		}
		
	//set Save Prefs button functionality
	usavebtn.onClick=function()
		{
		if (upreftype.text=="PSD")
			{
			storeallinfo();
			savepsdprefs();
			saveglobalprefs();
			}
		else
			{
			storeglobalinfo();
			saveglobalprefs();
			}
		alert("The preferences were saved.","CONFIRMATION");
		}
	
	// set save images button functionality
	uusebtn.onClick=function()
		{
		// if the psd isn't save then the command line functionality won't be passed the psd name or psd path when Use PSD.. is checked
		// the forcepsd variable is used for a workaround.
		forcepsd=true;
		storeglobalinfo();
		mainwindow.enabled=false;
		savepsd();
		//if the Sets option is set to ignore marked layer sets run tgamagic, if it isn't then run setupsets.
		if (elyrsets != 0) setupsets();
		if (elyrsets == 0) tgamagic();
		mainwindow.close(4);
		}	
		
	utgabtn.onClick=function()
		{
		forcepsd=false;
		storeglobalinfo();
		mainwindow.enabled=false;
		if (elyrsets != 0) setupsets();
		if (elyrsets == 0) tgamagic();
		mainwindow.close(4);
		}
	
	// if the script is being called from vQuickPSDTGA.jsx then we don't need to open the UI
	// (Setting it up was still required to correctly process certain variables)
	// The call from vQuickPSDTGA is identified if "goteki" is true, 
	// if "goteki" is undefined (causing an error) then this script was not eval'd from that
	// script, and the UI needs to be opened.
	
	// to debug, comment out from here to the { after catch(err), and the following }

	try
		{
		if(goteki==true)
			{
			psdtoui();
			storeglobalinfo();
			if (elyrsets != 0) setupsets();
			if (elyrsets == 0) tgamagic();
			}
		}
		
	catch(err)
		{
		//open the window
		mainwindow.center(); mainresult=mainwindow.show();
		//if cancel button was pressed
		if (mainresult==4) return;
		}

	}

/*
------------
FUNCTION: setupsets
------------
*/	
function setupsets()
	{
	// set changemenot to false so that TGA Magic doesn't change the name we'll be sending to it
	changemenot=false;
	// get the base TGA name for saving
	createtganame();
	// store that result so we can get it back later
	tganamestore=tganame;
	// set up a storage array for the tidle marked layersets
	var tsets = new Array();
	//set up an array of those T layersets that are currently visible
	//this is so we can restore the PSD to the state it was when we entered the script later
	//we don't need a list of layersets not visible, since if they're not visible, their visibility is already where it should be.
	var vsets = new Array();
	//set up a storage array for those layers that are both visible and marked, so we can append an initial just to the current set.
	// yes, it's an amusing variable name too!
	var tvsets = new Array();
	i=(thepsd.layerSets.length-1);
	if (i == -1) 
		{
		alert("This document does not appear to have any Layersets.\nRerun the script and ensure the Sets option is as you want it in the script window.","ERROR: Parameters Not Met"); 
		return;
		}
	// loop until the count of i is the same as the number of layersets
	while (i != -1)
		{
		// define the current layerset (as number i) as "namelayer"
		var namelayer=thepsd.layerSets[i];
		// check if the layer is visible, if it is, add its index to the visible list
		if (namelayer.visible==true) vsets[vsets.length]=i;
		// search the layers name for the "~" symbol
		var tiset = namelayer.name.indexOf(esetmark);
		// if the ~ symbol was found as the first character...
		//set the last entry in the tsets array to be i (the current layerset index)
		if (tiset == 0)	
			{
			tsets[tsets.length]=i;
			//check to see if it's visible (again), if it is then hide it (so we don't have any ~ marked sets visible to begin)
			if (namelayer.visible==true) 
				{
				namelayer.visible=false;
				//also, add the current layer number to tvsets, as we know it was marked, and it was visible when the script was run.
				tvsets[tvsets.length]=i;
				}
			}
		// reduce i by 1
		i--
		}
	// if there are no layers marked with a ~ then return
	if (tsets.length<1) 
		{
		alert("There were no layers correctly marked to save. \nRerun the script and ensure the Sets option is as you want it in the script window.","ERROR: Parameters Not Met");
		return;
		}
		
	if (elyrsets==1||elyrsets==2||elyrsets==3)
		{
		if (tvsets.length<1)
			{
			alert("There were no layers correctly marked and visible. \nRerun the script and ensure the Sets option is as you want it in the script window.","ERROR: Parameters Not Met");
			return;
			}
		}			
	//if the sets option requires all the layers saved, set i to be the number of tilde layers found
	//else set it to the number of visible tilde layers
	(elyrsets==4||elyrsets==5||elyrsets==6) ? i=tsets.length-1:i=tvsets.length-1;

	//loop until i = -1
	while (i != -1)
		{
		//store the current tilde layer number for this loop in tempstore, or store the highest previously visible one, depending on sets options
		
		(elyrsets==4||elyrsets==5||elyrsets==6) ? tempstore=tsets[i]:tempstore=tvsets[i];
	
		// set namelayer to be the current loop's tilde layer, and then make it visible. 
		namelayer=thepsd.layerSets[tempstore];
		namelayer.visible=true;
		// run getsetname to get the name for the TGA, and then in turn to run TGA magic...
		getsetname(namelayer.name,tempstore);
		tgamagic();
		// hide the layer again
		namelayer.visible=false;
		// reduce i by 1 is saving all marked sets, or set it to -1 if just the one set, completing the loop.
		(elyrsets==4||elyrsets==5||elyrsets==6) ? i--:i=-1;
		}
		
	//now to return the layersets' visibility to the same as we started with.
	//set i to be the number of originally visible layers
	i=vsets.length-1;
	while (i != -1)
		{
		namelayer=thepsd.layerSets[vsets[i]];
		namelayer.visible=true;
		i--
		}
	}


/*
------------
FUNCTION: getsetname
------------
*/	
function getsetname(layername,loop)
	{
	// set name clip to 1.  If the ~ is immediately followed by the name then this will clip the name in the right place
	var nameclip=1;
	// see if the tilde is followed by a space before the name, if it is then nameclip is set to 2 to include the space 
	var mrspace=layername.indexOf(" ");
	if (mrspace==1) nameclip=2;

	// set namestore to be the rest of the setname after the ~ and the space (if any) are removed. 
	var namestore=layername.substring(nameclip,layername.length);
	
	//check if the layer name has a second tilde at the end that would denote it being a normal map
	
	var normality = 0;
	var mrtilde=namestore.indexOf(esetmark);
	if (mrtilde != -1) 
		{
		//set normality to 1, to show this set requires Normals processing
		normality=1;
		//set up how many characters need to be stripped off the end of the name 1 for just ~, 2 for a space as well.
		nameclip = 1;
		mrspace=namestore.indexOf(" "+esetmark);
		if (mrspace != -1) nameclip = 2;
		//strip the name of the characters we don't need.
		var namestore=namestore.substring(0,(namestore.length-nameclip));
		}
	
	//Store the name in another variable for use with alpha channel searching, as we may need to mess around with namestore.
	alphaname=namestore;
		
	// if we're using option 1 or 4 on the sets dialogue, we need to append the tganame with the relevant initial 
	if (elyrsets==1||elyrsets==4)
		{
		tganame=tganamestore;
		namestore="_"+(namestore.substring(0,1));		
		tganame=tganame+namestore+".tga";
		}
	//if we are using option 2 or 5, the layer name is already the intended file name 
	if (elyrsets==2||elyrsets==5)
		{
		tganame=namestore+".tga";
		}
	//option 3 or 6 will append the layer's name onto the predefined tganame
	if (elyrsets==3||elyrsets==6)
		{
		tganame=tganamestore+namestore+".tga";
		}
	//if our layerset has been flagged as a normal map then activate the normal map conversion, else turn it off (even if it's set to on in the prefs)
	if (normality==1)
		{ 
		(enmsave > 0) ? enmsave=unormalmap.selection.index : enmsave=1;
		}
	else
		{ enmsave=0; }
	}

/*
------------
FUNCTION: savepsd
------------
*/	
function savepsd()
	{
	// see if the image has previously been saved as a PSD (if there's no file extension then it hasn't been)
	if (psdname.indexOf(".psd") <= 0)
		{
		//if it wasn't previously saved then we need to know where the user wants to put it
		psdsaved=false;
		//let the user know what's going on
		alert("The current image must be saved as a PSD file before continuing.","WARNING");
		//set "psdsave" to be the users "My Pictures" Folder, and add the name stored in "psdname".  
		var psdsave=File("~/My%20Documents/My%20Pictures/"+psdname);
		//open the save dialogue, using the predefined filename and path as the default
		psdpath=psdsave.saveDlg("Save as .psd...","Valid Types:*.psd;*.tga,All Files:*");
		//if the user clicked cancel then exit
		if(psdpath==null) {psdsaved=false; return;}
		// this will end in .psd no matter what, so we need to remove any extension info that may have been added
		//in the save dialogue first. 
		// Convert the file path to a workable string
		psdpath=psdpath.toString();
		//now find out how many characters there are before the full stop 
		psdsave=psdpath.indexOf(".");
		//then we copy the string back into itself, up to the full stop and bolt ".psd" on the end
		psdpath=psdpath.substring(0,psdsave)+".psd";
		//store the path string somewhere useful
		psdname=psdpath;
		//and convert psdpath back into a filepath object
		thepath=File(psdpath)
		}
	else {psdsaved=true};
	if (psdsaved==true)
		{
		//set "thepath" to be the document's path.
		thepath=thepsd.path;
		//save the psd and mark it as such before returning
		try
			{			
			thepsd.save();
			}
		catch(err)
			{
			alert ("The PSD could not be saved.\nPossibly it is checked in, or otherwise\nwrite protected, or another error occured.\nThe TGA may still be saved.","ERROR: Unable to Write");
			psdsaved=false;
			}
		}
	else
		{
		try
			{
			thepsd.saveAs(thepath, PhotoshopSaveOptions, false, Extension.NONE);
			psdname=activeDocument.name;
			psdsaved=true;
			}
		catch(err)
			{
			alert ("The PSD could not be saved.\nPossibly it is checked in, or otherwise\nwrite protected, or another error occured.\nThe TGA may still be saved.","ERROR: Unable to Write");
			psdsaved=false;
			}		
		}
	}

/*	
------------
FUNCTION: createtganame
------------
*/
function createtganame()
	{
	//check if the user wanted to save with the same name as the PSD or not
	if (epsdname==true)
		{
		//set "tganame" to be the name of the currently active document
		tganame=activeDocument.name		
		//if changemenot is true then we're not saving sets, so the name we now have is from a copy of the PSD
		if (changemenot==true)
			{
			//since the name will end in " copy", search for the string, and return its position to "tgatemp"
			var tgatemp=tganame.indexOf(" copy");
			//using this position, extract all of the name prior to " copy" and replace "tganame" with it
			tganame=tganame.substring(0,tgatemp);
			tganame=tganame+".tga";
			}
		// if changemenot was false we need to remove the .psd from the end of the filename in the same manner
		else
			{
			var tgatemp=tganame.indexOf(".psd");
			//using this position, extract all of the name prior to ".psd" and replace "tganame" with it
			tganame=tganame.substring(0,tgatemp);
			}
		}
	else
		{
		//do basically the same thing as above, but substitute the users TGA name for the TGA's own
		var temp=etganame.indexOf(".");
		if (temp>=0) etganame=etganame.substring(0,temp);
		//if changemenot is true, append the name with ".tga" (if it is false this will be done in the sets loop)
		(changemenot==true) ? tganame=etganame+".tga":tganame=etganame;
		}
	}
	
/*	
------------
FUNCTION: tgamagic
------------
*/
function tgamagic()
	{
	//set up the basics
	
	//set "thetga" to be a duplicate of "thepsd" (this will automagically make the duplicate active)
	thetga=thepsd.duplicate()

	//do anything regarding alphas
	alphamagic();
	
	//recheck the channels length, so we know if we have any alpha channels left
	chanlength=thetga.channels.length;
	//if we still have an alpha by this stage we'll be wanting to save it:
	//see if we do, and then set the image bits per pixel to save an alpha or not based on the result.
	if (chanlength>3)
		{
		TargaSaveOptions.alphaChannels=true;
		TargaSaveOptions.resolution=TargaBitsPerPixels.THIRTYTWO;		
		}
	else 
		{
		TargaSaveOptions.alphaChannels=false;
		TargaSaveOptions.resolution=TargaBitsPerPixels.TWENTYFOUR;
		}
	
	//flatten the tga
	thetga.flatten();	
	//did the user want the image rescaled - if so, run resizemagic()
	if ((eimgx>0)||(eimgy>0)) resizemagic();
	// did the user want to save as a normal map?  If so run normalmapmagic()
	if (enmsave>0) normalmapmagic();
	
	//check to see if the script is coming from getsetsname, if it is we don't need to change the TGA name.
	if (changemenot==true)
		{
		//else we need to change the tga name to a useable format
		createtganame();
		}
		
	//if the user chose to save the tga to the same folder as the PSD do this 
	if (epsdpath==true)
		{	
		//suffix the name with thepsd's path and convert from a string to a filepath
		thepath=thepsd.path;
		tgapath=File(thepath+"/"+tganame);
		}
	else
		{
		//suffix the name with the path the user chose, and convert from a string to a filepath
		tgapath=File(etgapath+"/"+tganame);
		}
	
	savetga();
	
	activeDocument=thepsd;
	
	}
	
/*
------------
FUNCTION: savetga
------------
*/	
function savetga()
	{
	//set up the TGA options to reflect the compression setting
	(ecompress==false) ? TargaSaveOptions.rleCompression=false:TargaSaveOptions.rleCompression=true;

	//save the tga
	try
		{
		thetga.saveAs(tgapath, TargaSaveOptions, false, Extension.NONE);
		tgasaved=true;
		}
	catch(err)
		{
		alert ("The TGA could not be saved.\nPossibly it is checked in, or otherwise\nwrite protected, or another error occured.","ERROR: Unable to Write");
		tgasaved=false;
		}
	
	//Whether the TGA remains open or closed, set "thepath" to be it's path for when/if we callcommand.
	thepath=thetga.path;
	
	//close the tga duplicate if the user did not choose to leave it open
	if (etgaopen==false) thetga.close(SaveOptions.DONOTSAVECHANGES);
	if (ecmdrun==true&&tgasaved==true) callcommand();
	}
	
/*	
------------
FUNCTION: alphamagic
------------
*/
function alphamagic()
	{
	var actchan
	var thechannel

	//set chanlength to the length of the channels array
	chanlength=thetga.channels.length;
	//lop one off the end, so that chanlengh is a useable number for referencing the array
	chanlength--;
	//set actchan to reference the last channel in the array
	actchan=thetga.channels[chanlength];
	
	// if the image requires no alpha (remove all channels), or highest alpha 
	// (remove all but one channel) do this:
	if (esavealp == 1||esavealp == 0||esavealp > 2)
		{
		//setup the var channum, and place in it the number of channels required remaining after the loop.
		//Note that the number is the number of alphas +2 for R(0),G(1),B(2).
		var channum;
		(esavealp == 0||esavealp > 2) ? channum=2:channum=3;
		
		while (chanlength>channum)
			{
			//set thechannel to refer to the active channel, and set the active channel to actchan
			thetga.activeChannel=thetga.channels[actchan.name];
			thechannel=thetga.activeChannel;
			//delete the active channel
			thechannel.remove();
			//alter the channel that will be made active to the next one up the list
			chanlength--
			actchan=thetga.channels[chanlength];
			}
		}
	
	//if the alpha is to be defined by the highest visible marked Layerset, do this:
	if (esavealp == 2)
		{
		//set up a variable to avoid duplicate name issues - once the name is found chanfound becomes 1
		var chanfound=0;
		while (chanlength>2)
			{
			//set thechannel to refer to the active channel, and set the active channel to actchan
			thetga.activeChannel=thetga.channels[actchan.name];
			thechannel=thetga.activeChannel;
			var channame = thechannel.toString()
			
			//search channame for the name stored in alphaname
			var chantemp=channame.indexOf(alphaname);
			
			//if it's not found, or chanfound==1, delete it
			if  (chantemp == -1||chanfound==1)
				{
				//delete the active channel
				thechannel.remove();
				}
			//if it is found then set chanfound to 1 before moving onto the next channel
			else
				{
				chanfound=1;
				}
			//alter the channel that will be made active to the next one up the list
			chanlength--
			actchan=thetga.channels[chanlength];
			}
		}
	
	if (esavealp > 2)
		{
		createnewalpha();
		}
	}

/*
------------
FUNCTION: createnewalpha
Creates a new alpha channel, or simply a chromakey background, based on the images transparency
------------
*/	
function createnewalpha()
	{
	
	//check to ensure we're on a visible layer, and if not to move through the layerset index until we find one
	//this is so we can merge visible.
	i=(thetga.layers.length-1);

	// loop until the count of i is the same as the number of layers (which it may be early if it gets to a visible one)
	while (i != -1)
		{
		//select the layer with the same index as i
		var visilayer=thetga.layers[i];
		//if the layer is visible, do this
		if (visilayer.visible==true) 
			{
			//make the currently selected layer the active one
			thetga.activeLayer=visilayer;
			//set i to 0 so the next while check will exit out of the loop.
			i=0;
			}
		// reduce i by 1
		i--
		}
		
	//merge visible layers
	mergevisible();
	
	// change the name of the current layer to worklayer, and set the variable thelayer to refer to it
	var thelayer=thetga.activeLayer;
	thelayer.name="WorkLayer";
	
	// the following will duplicate the layer ontop of itself ~80 times, removing any transparency in the process
	// it will then merge the layers so we have one layer with no transparent pixels.  
	// Obviously we don't want to do this if the user wants an antialiased alpha
	
	if (esavealp>3)
		{
		//set up some variables to contol the following loops
		var i=0;
		var ii;
		
		//start the primary loop
		while (i<8)
			{
			ii=0;
			//start the secondary loop to duplicate the layer 5 times
			while (ii<6)
				{
				//copy thelayer
				thelayer.duplicate(thelayer,ElementPlacement.PLACEBEFORE);
				ii++;
				}
			//merge the layers with the mergelayers function
			mergevisible()
			i++;
			}
		}
	
	if (esavealp<5)
		{
		//create an alpha channel from this new merged layer
		//call the layerselectionfunction()
		layerselection();
		//we need to create a new channel here, and then store the selection into it...
		var userchannel=thetga.channels.add();
		userchannel.name="saveme";
		thetga.selection.store(userchannel,SelectionType.REPLACE);
		}
		
	//create a new layer, and stick it under the layer we just created 	
	var baselayer=thetga.artLayers.add();
	baselayer.move(thelayer,ElementPlacement.PLACEAFTER);
	//define a solid colour
	try
		{
	var backcol=new SolidColor();
	backcol.rgb.hexValue=ehexcol;
		}
	catch(err)
		{
		alert("invalid hex value!")
		}
	//fill the new layer with the solid colour
	thetga.selection.selectAll()
	thetga.selection.fill(backcol);
	thetga.selection.deselect()
	}

/*
------------
FUNCTION: resizemagic
------------
*/	
function resizemagic()
	{
	//get the current image sizes
	var iheight=thetga.height;
	var iwidth=thetga.width;
	//enumerate the resize method selection
	eresmet=parseInt(eresmet);
	//setup an array of possible sizes (as Unitvalues)
	resizeh=new Array(iheight,UnitValue("64 px"),UnitValue("128 px"),UnitValue("256 px"),UnitValue("512 px"),UnitValue("1024 px"),UnitValue("2048 px"));
	resizew=new Array(iwidth,UnitValue("64 px"),UnitValue("128 px"),UnitValue("256 px"),UnitValue("512 px"),UnitValue("1024 px"),UnitValue("2048 px"));
	var resmet=new Array(ResampleMethod.NONE,ResampleMethod.BICUBIC,ResampleMethod.BICUBICSHARPER,ResampleMethod.BICUBICSMOOTHER,ResampleMethod.BILINEAR,ResampleMethod.NEARESTNEIGHBOR);
	//rescale the image
	thetga.resizeImage(resizeh[eimgx],resizew[eimgy],72.00,resmet[eresmet]);
	}
	
/*
------------
FUNCTION: normalmapmagic
------------
*/

function normalmapmagic()
	{
	// If the type is set to Auto, work out if the image is already a normal map to decide which filter type to run
	if (enmsave==1)
		{
		//select all the standard familiar NM blue colour.
		var id7812 = charIDToTypeID( "ClrR" );
		var desc1573 = new ActionDescriptor();
		var id7813 = charIDToTypeID( "Fzns" );
		desc1573.putInteger( id7813, 20 );
		var id7814 = charIDToTypeID( "Mnm " );
		var desc1574 = new ActionDescriptor();
		var id7815 = charIDToTypeID( "Lmnc" );
		desc1574.putDouble( id7815, 58.360000 );
		var id7816 = charIDToTypeID( "A   " );
		desc1574.putDouble( id7816, 25.780000 );
		var id7817 = charIDToTypeID( "B   " );
		desc1574.putDouble( id7817, -64.510000 );
		var id7818 = charIDToTypeID( "LbCl" );
		desc1573.putObject( id7814, id7818, desc1574 );
		var id7819 = charIDToTypeID( "Mxm " );
		var desc1575 = new ActionDescriptor();
		var id7820 = charIDToTypeID( "Lmnc" );
		desc1575.putDouble( id7820, 58.360000 );
		var id7821 = charIDToTypeID( "A   " );
		desc1575.putDouble( id7821, 25.780000 );
		var id7822 = charIDToTypeID( "B   " );
		desc1575.putDouble( id7822, -64.510000 );
		var id7823 = charIDToTypeID( "LbCl" );
		desc1573.putObject( id7819, id7823, desc1575 );
		executeAction( id7812, desc1573, DialogModes.NO );
				
		//find out this selected enough of the colour to qualify as a normal map:

		// Because of a glitch in CS2 we need to use this round about method to find the size of the selection
		// Many thanks go to xbytor of ps-scripts.com for figuring this one out.
		/*
		Copyright: (c)2006, xbytor 
		License: http://creativecommons.org/licenses/LGPL/2.1 
		Contact: xbytor@gmail.com
		*/
		
		// First, was anything selected at all?
		var nmselect = false; 
		// store a history state
		var as = thetga.activeHistoryState;
		// deselect any existing selection 
		thetga.selection.deselect(); 
		// if there's a difference between the image an the history state then there was a selection.  
		// Restore the history state and set nmselect to true, otherwise just set enmsave to 2 so 
		// the bump to normal conversion runs.
		 
		if (as != thetga.activeHistoryState)
			{
			nmselect = true; 
			thetga.activeHistoryState = as;
			}
		else 
			{
			enmsave=2;
			}
		
		//The following turned out not to be needed, but I'm keeping it here as it's a handy bit of code :)
		/*	
		//Check if there was a selection, calculate how much *was selected, and if it's 
		// more than a preset amount set enmsave to 3 to run just the normalize filter 
		if (nmselect == true)
			{
			// add a new layer and fill it (filling only the selected area), 
			// then calculate the boundary of the layer, then use history to return to the pre selected state.   
			nmselect = thetga.artLayers.add(); 
			thetga.selection.fill(app.foregroundColor); 
			nmselect = nmselect.bounds; 
			thetga.activeHistoryState = thetga.historyStates[thetga.historyStates.length-4];
			//calcualte the number of pixels selected from the boundaries.
			nmselect=nmselect[2]*nmselect[3];
			//set enmsave to the correct type based on the findings.
			(nmselect > 128) ? enmsave=3:enmsave=2;
			}
		*/
		}
	
	try
		{
		if (enmsave==2)
			{
			//invert the depth if the user chose too
			if (enminv==true) enmdpth=0-enmdpth;
			//run the nVidia plugin (standard)
			var id298 = charIDToTypeID( "sdk1" );
		    var desc56 = new ActionDescriptor();
		    var id299 = charIDToTypeID( "Amnt" );
		    var id300 = charIDToTypeID( "#Prc" );
		    //this is where the depth is defined
		    desc56.putUnitDouble( id299, id300, enmdpth );
		    var id301 = charIDToTypeID( "filT" );
		    var id302 = charIDToTypeID( "typT" );
		    var id303 = charIDToTypeID( "fil4" );
		    desc56.putEnumerated( id301, id302, id303 );
		    var id304 = charIDToTypeID( "conV" );
		    var id305 = charIDToTypeID( "typC" );
		    var id306 = charIDToTypeID( "cnv1" );
		    desc56.putEnumerated( id304, id305, id306 );
		    var id307 = charIDToTypeID( "alph" );
		    var id308 = charIDToTypeID( "typA" );
		    var id309 = charIDToTypeID( "alp1" );
		    desc56.putEnumerated( id307, id308, id309 );
		    //this is where the wrap option is defined
		    var id310 = charIDToTypeID( "wrap" );
		    desc56.putBoolean( id310, enmwrap );
			executeAction( id298, desc56, DialogModes.NO );
			}
		if (enmsave==3)
			{
			//for normalize only:
			var id7 = charIDToTypeID( "sdk1" );
		    var desc3 = new ActionDescriptor();
		    var id8 = charIDToTypeID( "Amnt" );
		    var id9 = charIDToTypeID( "#Prc" );
		    desc3.putUnitDouble( id8, id9, 2.000000 );
		    var id10 = charIDToTypeID( "filT" );
		    var id11 = charIDToTypeID( "typT" );
		    var id12 = charIDToTypeID( "fil4" );
		    desc3.putEnumerated( id10, id11, id12 );
		    var id13 = charIDToTypeID( "conV" );
		    var id14 = charIDToTypeID( "typC" );
		    var id15 = charIDToTypeID( "cnv8" );
		    desc3.putEnumerated( id13, id14, id15 );
		    var id16 = charIDToTypeID( "alph" );
		    var id17 = charIDToTypeID( "typA" );
		    var id18 = charIDToTypeID( "alp1" );
		    desc3.putEnumerated( id16, id17, id18 );
			executeAction( id7, desc3, DialogModes.NO );
			}
		}
	catch(err)
		{
		alert("There was a problem with the normal map generation/nPossibly the nVidia plugin is not availale, or is the wrong version")
		} 
	}
	
/*
------------
FUNCTION: callcommand
------------
*/
function callcommand()
	{
	//set up a "safe" variable for the command line data (this probably isn't required, but just to be on the safe side...)
	var cmdtxt=ecmdline;
	//set up a storage variable for the file system version of the path
	var temppath = thepath.fsName+"\\";
	
	//set up two arrays - one with the external alias' of the TGA and Path, and one with the internal.
	//this allows us to use the same chunk of code to find and replace both of them in the command line
	var wordage = new Array("<FILENAME>","<FILEPATH>");
	var varname = new Array(tganame,temppath);
	// loop through the following twice - once for the tga name, once for the path
	for (j=0; j<2; j++)
		{
		// set up "locate" with a number higher than -1.  This is so the script doesn't bomb out immediately (see below)
		var locate = 1;
		// keep doing the following loop until the string isn't found in the command line (returns -1)
		while (locate > -1)
			{
			// find the string (either <FILENAME> or <FILEPATH> depending on the value of i) 
			locate = cmdtxt.indexOf(wordage[j]);
			// if it found the string, do this (else just exit this itteration of the for loop)
			if (locate > -1)
				{
				// strip out the start of the command line up to the current search string
				var stringstart = cmdtxt.substring(0,locate);
				// strip out the command line after the search string
				var stringend = cmdtxt.substring(locate+10,cmdtxt.length);
				// combine the strings, and insert the variable matching the alias between them
				cmdtxt=stringstart+varname[j]+stringend;
				}
			}
		}	
	// create a batchfile containing the command line and execute it (PC only obviously).
	var bat = new File(prefpath + "/vSavePSDTGA.bat"); 
  	bat.open("w"); 
  	bat.writeln("@echo off");
  	bat.writeln(cmdtxt);
 	bat.writeln("echo vSavePSDTGA Batch Process...")
  	bat.writeln("exit");
  	bat.close(); 
  	bat.execute();
	}
	
/*
------------
FUNCTION: validhexcol
------------
*/

function validhexcol(input)
	{
	//This function takes the output from the system colorpicker and converts it to a valid hexadecimal value
	//convert the output to a hex value
	var hexvalue=input.toString(16).toUpperCase();
	//this will strip out leading 0's leaving an invalid value
	//if there are less than 6 characters fill the string with 0's until it reaches 6 - a valid hex value
	while (hexvalue.length<6)
		{
		hexvalue="0"+hexvalue;
		}
	//return the value
	return(hexvalue);
	}

function colorchange()
		{
		//This function converts the hex value from the hext text field into a color swatch that fills the adjacent panel
		//break the hex string into the hex values for red, green and blue
		var redval = ualphexcol.text.substring(0,2);
		var grnval =ualphexcol.text.substring(2,4);
		var bluval = ualphexcol.text.substring(4,6);
		//convert each of these into a decimal number
		redval=parseInt(redval,16);
		grnval=parseInt(grnval,16);
		bluval=parseInt(bluval,16);
		//now convert the decimal representation from a number between 0 and 255 to a value between 0 and 1
		// to do this we find the percentage of the value from 255, round it, and then / by 100
		redval=(Math.round(redval/255*100))/100;
		grnval=(Math.round(grnval/255*100))/100;
		bluval=(Math.round(bluval/255*100))/100;
		
		//create an array containing these values, and the opacity of the swatch (always 1)
		var defcolor=new Array();
		defcolor = [redval,grnval,bluval, 1];
		
		// create a variable representing the swatch panel's graphics aspect 
		var graph=ualpkey.graphics;
		// create a "brush" with which to fill the swatch panel's background
		// format is elements graphics aspect.newBrush (element's brushtype, previously defined color array, 1 for some reason I can't yet work out)
		var brush =graph.newBrush (graph.BrushType.SOLID_COLOR, defcolor,1)
		//fill the swatch panel
		graph.backgroundColor = brush;
		}

/*
------------
FUNCTION: mergevisible
------------
*/
function mergevisible()
	{
	var id1014 = charIDToTypeID( "MrgV" );
	executeAction( id1014, undefined, DialogModes.NO );
	}
	
/*
------------
FUNCTION: layerselection
------------
*/
function layerselection()
	{
	//script listener output to select an area based on a layers transparency	
	var id5407 = charIDToTypeID( "setd" );
    var desc1010 = new ActionDescriptor();
    var id5408 = charIDToTypeID( "null" );
    var ref675 = new ActionReference();
    var id5409 = charIDToTypeID( "Chnl" );
    var id5410 = charIDToTypeID( "fsel" );
    ref675.putProperty( id5409, id5410 );
    desc1010.putReference( id5408, ref675 );
    var id5411 = charIDToTypeID( "T   " );
    var ref676 = new ActionReference();
    var id5412 = charIDToTypeID( "Chnl" );
    var id5413 = charIDToTypeID( "Chnl" );
    var id5414 = charIDToTypeID( "Trsp" );
    ref676.putEnumerated( id5412, id5413, id5414 );
    desc1010.putReference( id5411, ref676 );
	executeAction( id5407, desc1010, DialogModes.NO );
	}




