// SCRIPT NAME: vScriptListener
var VERSION=1.1;
// AUTHOR: James A. Taylor
// DESCRIPTION: Controls aspects of the Photoshop script listener output.

#target photoshop
app.bringToFront();

// GLOBAL VARIABLES
var i, extension, plugstore, pluglocation, jslog, vblog, jssize, vbsize;

// START THE SCRIPT
main();

//FUNCTIONS

/*
------------
FUNCTION: main
------------
*/
function main()
	{
	//set up the log file paths
	var logpath = Folder.desktop;
	jslog= new File(logpath+"/ScriptingListenerJS.log");
	vblog= new File(logpath+"/ScriptingListenerVB.log");
	
	//see if the plugin is installed:
	plugpath=Folder.startup;
	var plugpath = new Folder(plugpath+"/Plug-Ins/");
	checklistener(plugpath,"ScriptListener.8li"); 
	
	//chose what to do based on whether the plugin was found in the plugin's tree:
	if (plugstore.length<1)
		{
		installlistener();
		}
	else
		{
		//store the plugin's location
		pluglocation=plugstore[0];
		//call the user interface
		UI();
		}

	}
	
/*
------------
FUNCTION: UI
------------
*/
function UI()
	{
	
	setupui =
		"dialog {text: 'vScriptListener', orientation:'column' \
			, optgrp: Group {orientation:'row', alignment:'center' \
				, optgrpjs: Panel {orientation:'column', alignment:'center' \
					, jstxt: StaticText {text:'JS: UNLOCKED', preferredSize:[100,21]} \
					, jslock: Button {text:'Lock Output', preferredSize:[100,21]} \
					, jsclear: Button {text:'Clear Logfile', preferredSize:[100,21]} \
					, jsopen: Button {text:'Open Logfile', preferredSize:[100,21]}\
					, jssize: StaticText {text:'Size: 0MB', preferredSize:[100,21]}\
				}\
				, optgrpvb: Panel {orientation:'column', alignment:'center' \
					, vbtxt: StaticText {text:'VB: UNLOCKED', preferredSize:[100,21]} \
					, vblock: Button {text:'Lock Output', preferredSize:[100,21]} \
					, vbclear: Button {text:'Clear Logfile', preferredSize:[100,21]} \
					, vbopen: Button {text:'Open Logfile', preferredSize:[100,21]}\
					, vbsize: StaticText {text:'Size: 0MB', preferredSize:[100,21]}\
				}\
			}\
			, btngroup: Group { orientation:'row', alignment:'center', alignclildren:'center' \
				, okbtn: Button { text:'DONE', properties:{name:'OK'} } \
			}\
		}";
	
	//initialise window
	mrwindow=new Window(setupui);
	mrwindow.text=mrwindow.text+" v"+VERSION;
	//set up UI references

	statusjs=mrwindow.optgrp.optgrpjs.jstxt;
	lockjs=mrwindow.optgrp.optgrpjs.jslock;
	clearjs=mrwindow.optgrp.optgrpjs.jsclear;
	openjs=mrwindow.optgrp.optgrpjs.jsopen;
	sizejs=mrwindow.optgrp.optgrpjs.jssize;
	
	statusvb=mrwindow.optgrp.optgrpvb.vbtxt;
	lockvb=mrwindow.optgrp.optgrpvb.vblock;
	clearvb=mrwindow.optgrp.optgrpvb.vbclear;
	openvb=mrwindow.optgrp.optgrpvb.vbopen;
	sizevb=mrwindow.optgrp.optgrpvb.vbsize;
	
	aok=mrwindow.btngroup.okbtn;
	
	//ensure the log files exist - if they don't, make them.
	if (jslog.exists==false)
		{
		//create the file
		jslog.open("w");
		//close the file
		jslog.close;
	 	}
	 if (vblog.exists==false)
		{
		//create the file
		vblog.open("w");
		//close the file
		vblog.close;
	 	}
	
	//fill in the size texts
	getlogsize();
	sizejs.text="Log Size: "+jssize+"MB";
	sizevb.text="Log Size: "+vbsize+"MB";
	
	//set the correct startup text and state of the lock/unlock buttons
	switchstates(jslog,lockjs,clearjs,statusjs,"JS: ");
	switchstates(vblog,lockvb,clearvb,statusvb,"VB: ");
	
	//set the lock/unlock button functionalities for JS and VB
	//JS:
	lockjs.onClick=function()
		{
		//switch the R/W flag of the file
		jslog.readonly = !jslog.readonly;
		switchstates(jslog,lockjs,clearjs,statusjs,"JS: ");
		}
	//VB:
	lockvb.onClick=function()
		{
		vblog.readonly = !vblog.readonly;
		switchstates(vblog,lockvb,clearvb,statusvb,"VB: ");
		}
	//Set the clear buttons functionalities
	//JS: 
	clearjs.onClick=function()
		{
		//ensure this is what the user wants
		if (confirm("This will clear all data from the JS Script Listener log file!\n\nAre you sure you wish to do this?",false,"What to do?")==false) return;
		//open and clear the file
		jslog.open("w");
		//close the file
		jslog.close;
		//adjust the listed filesize to the new value (0 MB)
		sizejs.text="Log Size: 0MB";
		}
	//VB:
	clearvb.onClick=function()
		{
		//ensure this is what the user wants
		if (confirm("This will clear all data from the VB Script Listener log file!\n\nAre you sure you wish to do this?",false,"What to do?")==false) return;
		//open and clear the file
		vblog.open("w");
		//close the file
		vblog.close;
		//adjust the listed filesize to the new value (0 MB)
		sizevb.text="Log Size: 0MB";
		}
		
	//set the open file functionalities
	//JS:
	openjs.onClick=function()
		{
		jslog.execute();
		}
	//VB:
	openvb.onClick=function()
		{
		vblog.execute();
		}
		
	//open window
	mrwindow.center(); result=mrwindow.show();
	}
	
/*
------------
FUNCTION: switchstates
------------
*/
function switchstates(log,locktxt,clear,statext,lang)
	{
	if (log.readonly==true)
		{
		locktxt.text="Unlock Output";
		statext.text=lang+"LOCKED";
		clear.enabled=false;
		}
	else
		{
		locktxt.text="Lock Output";
		statext.text=lang+"UNLOCKED";
		clear.enabled=true;
		}
	}

/*
------------
FUNCTION: getlogsize
------------
*/
function getlogsize()
	{
	//get the javascript logfile size in MB
	jssize=(jslog.length/1048576);
	jssize=jssize*100;
	jssize=Math.round(jssize);
	jssize = jssize/100;
	//get the VBscript logfile size in MB
	vbsize=(vblog.length/1048576);
	vbsize=vbsize*100;
	vbsize=Math.round(vbsize);
	vbsize = vbsize/100;
	}
	
/*
------------
FUNCTION: checklistener
------------
*/
function checklistener(searchpath,searchfile)
	{
	//set plugpath to be the current folder
	Folder.current=searchpath;
	//set up a storage array for the search results
	plugstore = new Array();
	//search for the given file
	recursivesearch(searchpath,searchfile);
	}
	
/*
------------
FUNCTION: recursivesearch
INPUTS: curpath: the path to begin searching in, plugfile: The file to search for.
Searches through the entire folder tree under the plug-ins folder and returns any instances of ScriptListener.8li
------------
*/	
function recursivesearch(curpath,plugfile)
	{
	//check the current folder for the given file (scriptlistener plugin)
	var pluglist=curpath.getFiles(plugfile);
	//if the file is found, store it's full path in the plugstore array
	for (var i = 0; i < pluglist.length; i++) 
		{ 
		plugstore[plugstore.length]=pluglist[i];
		} 
	//now search the current folder for other folders, and add the results to the folders variable 
	var folders = curpath.getFiles(function (f) { return f instanceof Folder; }); 
	//loop through the folders one by one, and search each of them the same as this one
	for (var i=0; i<folders.length; i++)
		{
		recursivesearch(folders[i],plugfile);
		}
	}
	
/*
------------
FUNCTION: UI
------------
*/
function installlistener()
	{
	//first we need to locate the plugin - it should be in the scripting guide folder somewhere.
	plugpath=Folder.startup;
	plugpath = new Folder(plugpath+"/Scripting Guide/");
	checklistener(plugpath,"ScriptListener.8li");
	//if the plugin wasn't found it can't be installed - give the user the bad news.
	if (plugstore.length<1)
		{
		alert("ScriptListener was not found in its expected location.\n Manually verify that the plugin resides in Photoshop's Scripting Guide\Utilities folder.\n If the file is missing you may need to reinstall Photoshop CS2 for Script Listener functionality.","ERROR: Required file not found");
		return;
		}
	
	//check the user actually wants scriptlistener installed and exit if they don't
	if (confirm("Script Listener is not currently installed for use.\nWould you like to install the plugin to a\nlocation where it can be activated for use?",false,"What to do?")==false) return;
	
	// copy the plugin to the plug-ins folder
	pluglocation=plugstore[0]
	plugpath=Folder.startup;
	plugpath = new File(plugpath+"/Plug-Ins/ScriptListener.8li");
	pluglocation.copy(plugpath);
	

	
	alert("The ScriptListener plugin has been installed to your Adobe Photoshop CS2\\Plug-Ins folder.\nTo activate Script Listener you will need to restart Photoshop.\n\nTo fully uninstall the plug-in you will need to delete it manually when Photoshop is not running.\nYou can block the plugin's output (effectively turning the plug-in off) by running this script again.","Operation Sucessfull");

	}