// SCRIPT NAME: vUVWHelper
var VERSION=1.3;
// AUTHOR: James A. Taylor
// COPYRIGHT: James A. Taylor
// DESCRIPTION: Creates UVW unwrap helper grids in a variety of styles.

#target photoshop
app.bringToFront();

//GLOBAL VARIABLES
var mrgrid,gridx,gridy;
var mrwindow,setupui,gridresult;
var ugsize,uisizex,uisizey,uorient,uhue,uok,ucan;
var gsize,isizex,isizey,orient,usehue;
var agsize, aisize;
var ofcol,obcol;
var myunit //A storage location for the current unit preference

//START THE SCRIPT
main();

//FUNCTIONS

/*
------------
FUNCTION: main
------------
*/
function main()
	{
	//store the default unit type and set the current preference to be pixels
	myunit = preferences.rulerUnits;
	preferences.rulerUnits = Units.PIXELS;
	//store the current back and fore colour as we'll be changing them
	obcol=backgroundColor;
	ofcol=foregroundColor;
	//call the user interface
	UI()
	//restore the back and fore colour to the ones we stored earlier
	backgroundColor=obcol;
	foregroundColor=ofcol;
	//now restore the default unit preference
	preferences.rulerUnits = myunit;
	}

function UI()
	{
	setupui =
		"dialog {text: 'vUVWHelper', orientation:'column' \
			, gridsizegroup: Group {orientation:'row', alignment:'left' \
				, gsizetext: StaticText {text:'Grid Size:'} \
				, gsize: DropDownList {properties:{items:['8x8','16x16','32x32','64x64','128x128']} } \
			}\
			, resizegroup: Group {orientation: 'row', alignment:'left' \
				, restext: StaticText {text:'Image Size:'} \
				, sizex: DropDownList {properties:{items:['128','256','512','1024']} } \
				, restext: StaticText {text:'x'} \
				, sizey: DropDownList {properties:{items:['128','256','512','1024']} } \
			}\
			, usetext: Checkbox {text:'Include Orientation Reference', alignment:'left', value:true} \
			, colourlayer: Checkbox {text:'Use Hueshift Background', alignment:'left', value:true} \
			, btngroup: Group { orientation:'row', alignment:'center', alignclildren:'center' \
				, okbtn: Button { text:'OK', properties:{name:'OK'} } \
				, clbtn: Button { text:'Cancel'} \
			}\
		}";
	//initialise window
	mrwindow=new Window(setupui);
	//setup window UI reference variables
	ugsize=mrwindow.gridsizegroup.gsize;
	uisizex=mrwindow.resizegroup.sizex;
	uisizey=mrwindow.resizegroup.sizey;
	uorient=mrwindow.usetext;
	uhue=mrwindow.colourlayer;
	uok=mrwindow.btngroup.okbtn;
	ucan=mrwindow.btngroup.clbtn;
	mrwindow.text=mrwindow.text+" v"+VERSION;
	
	//Set initial variable values
	ugsize.selection=1;
	gsize=16;
	uisizex.selection=2;
	isizex=512;
	uisizey.selection=2;
	isizey=512;
	orient=true;
	usehue=true;
	
	// set up size arrays
	agsize=new Array(8,16,32,64,128);
	aisize=new Array(128,256,512,1024);
	
	//Set gridsize functionality
	ugsize.onChange=function()
		{
		gsize=agsize[ugsize.selection.index];
		}
	//Set Image Size Functonality
	uisizex.onChange=function()
		{
		isizex=aisize[uisizex.selection.index];
		}
	uisizey.onChange=function()
		{
		isizey=aisize[uisizey.selection.index];
		}
	//Set checkbox functionality
	uorient.onClick=function() 
		{
		(orient==true) ? orient=false:orient=true;
		}
	uhue.onClick=function()
		{
		(usehue==true) ? usehue=false:usehue=true;
		}
	
	//Set Button Functionality
	uok.onClick=function(){mrwindow.close(4)};
	ucan.onClick=function(){mrwindow.close(2)};
	
	//open window
	mrwindow.center(); gridresult=mrwindow.show();
	if(gridresult==4) makesquare()
	}

/*
------------
FUNCTION: makesquare
------------
*/
function makesquare()
	{
	// make new document called "gridsquare", filled with white
	app.documents.add(gsize,gsize,72,"gridsquare",NewDocumentMode.RGB,DocumentFill.WHITE);
	//pause(720);
	// let that new doc be know as mrgrid
	mrgrid=activeDocument;
	//add a blank layer and call it "Grid"
	mrgrid.artLayers.add();
	mrgrid.layers[0].name="Grid";
	// set up a selection shape array 
	var selshape = [[1,1],[1,gsize],[gsize,gsize],[gsize,1]];
	// Set the selection as the defined shape
	mrgrid.selection.select(selshape);
	//invert the selection
	mrgrid.selection.invert();
	//et up a new solid colour called "black
	var black=new SolidColor();
	//define black as being... black
	black.rgb.hexValue="000000";
	//fill the selection with black
	mrgrid.selection.fill(black,ColorBlendMode.NORMAL,100);
	//lose the selection
	mrgrid.selection.deselect();
	if (orient==true) addorient();
	//flatten image
	mrgrid.flatten();
	mrgrid.selection.selectAll();
	mrgrid.selection.copy();
	mrgrid.close(SaveOptions.DONOTSAVECHANGES);
	pastegrid();
	//flatten final image
	Customgrid.flatten();
	}

/*
------------
FUNCTION: addorient
------------
*/	
function addorient()
	{
	//add another layer, and make it a text layer
	mrgrid.artLayers.add();
	mrgrid.layers[0].kind=LayerKind.TEXT;
	//set the font of the layer (which requires some special version of the name I guess)
	mrgrid.layers[0].textItem.font="ArialMT";
	//set the size in pixels
	if (gsize==8)
		{
		mrgrid.layers[0].textItem.size=6.00;
		}
	else
		{
		mrgrid.layers[0].textItem.size=(6.00*(ugsize.selection.index+1)*(ugsize.selection.index));
		}
	//Set the Text to appear (an R, as it doesn't mirror or flip in any way)
	mrgrid.layers[0].textItem.contents="R";
	//find out where the R was placed
	var bbox=mrgrid.layers[0].bounds;
	//invert the lower right X coord
	var tranmex=-bbox[2];
	//invert the lower right Y coord
	var tranmey=-bbox[3];
	//move the layer based on these values (which will make the lower right be at 0,0
	mrgrid.layers[0].translate(tranmex,tranmey);
	//move the layer into the desired location (12,12 on a 16x16 grid)
	switch(gsize)
		{
		case 8 : mrgrid.layers[0].translate(6,6); break;
		case 16 : mrgrid.layers[0].translate(12,12); break;
		case 32 : mrgrid.layers[0].translate(29,29); break;
		case 64 : mrgrid.layers[0].translate(59,59); break;
		case 128 : mrgrid.layers[0].translate(107,107); break;
		}
	}

/*
------------
FUNCTION: pastegrid
------------
*/
function pastegrid()
	{
	Customgrid=app.documents.add(isizex,isizey,72,"Custom Grid",NewDocumentMode.RGB,DocumentFill.WHITE);
	var mset = Customgrid.layerSets.add();
	var lset = mset.layerSets.add();
	var lay = lset.artLayers.add();
	lay.name="templayer";
	//Customgrid.layers[0].name="templayer";
	
	gridx=Customgrid.width;
	gridy=Customgrid.height;
	
	var gridxn=gridx.value/gsize;
	var gridyn=gridy.value/gsize;
	gridx.value=gsize
	gridy.value=0
	
	var i = 1;
	
	var selshape = [[0,0],[0,gsize-1],[gsize-1,gsize-1],[gsize-1,0]];
	Customgrid.selection.select(selshape)
	Customgrid.paste();
	//alert(gridx+","+gridy+","+gridxn+","+gridyn);
	
	while (i < gridxn)
		{
		lay=lay.duplicate(lay,ElementPlacement.PLACEBEFORE);
		lay.translate(gridx*i,gridy);
		lay=lay.merge();
		lay=Customgrid.activeLayer;
		i=i*2;
		}
	
	lay = lset.merge();
	i = 1
	
	while (i < gridyn)
		{
		lay=lay.duplicate(lay,ElementPlacement.PLACEBEFORE);
		lay.translate(gridy*i,gridx*i);
		lay=lay.merge();
		lay=Customgrid.activeLayer;
		i=i*2;
		}
		
	lay=mset.merge();
	lay.blendMode=BlendMode.MULTIPLY;
	mset = Customgrid.artLayers.add();
	mset.name = "colblend";
	mset.move(lay,ElementPlacement.PLACEAFTER); 
	
	if (usehue==true) colourness();
	}	

/*
------------
FUNCTION: colourness
------------
*/
function colourness()
	{
	var grey = new SolidColor();
	grey.rgb.hexValue="808080";
	Customgrid.selection.selectAll();
	Customgrid.selection.fill(grey,ColorBlendMode.NORMAL,100)
	Customgrid.selection.deselect();
	var white = new SolidColor();
	var black = new SolidColor();
	white.rgb.hexValue="ffffff";
	black.rgb.hexValue="000000";
	foregroundColor=black;
	backgroundColor=white;
	crossfill();
	}

/*
------------
FUNCTION: crossfill
------------
*/
function crossfill()
	{
	gridx=Customgrid.width;
	gridy=Customgrid.height;
	gridx=gridx.value;
	gridy=gridy.value;

	//===SELECT RED CHANNEL===
	var id108 = charIDToTypeID( "slct" );
	    var desc19 = new ActionDescriptor();
	    var id109 = charIDToTypeID( "null" );
	        var ref7 = new ActionReference();
	        var id110 = charIDToTypeID( "Chnl" );
	        var id111 = charIDToTypeID( "Chnl" );
	        var id112 = charIDToTypeID( "Rd  " );
	        ref7.putEnumerated( id110, id111, id112 );
	    desc19.putReference( id109, ref7 );
	executeAction( id108, desc19, DialogModes.NO );

	//===GRADIENT FILL RED===
	var id113 = charIDToTypeID( "Grdn" );
	    var desc20 = new ActionDescriptor();
	    var id114 = charIDToTypeID( "From" );
	        var desc21 = new ActionDescriptor();
	        var id115 = charIDToTypeID( "Hrzn" );
	        var id116 = charIDToTypeID( "#Pxl" );
	        desc21.putUnitDouble( id115, id116, 0 );
	        var id117 = charIDToTypeID( "Vrtc" );
	        var id118 = charIDToTypeID( "#Pxl" );
	        desc21.putUnitDouble( id117, id118, gridy );
	    var id119 = charIDToTypeID( "Pnt " );
	    desc20.putObject( id114, id119, desc21 );
	    var id120 = charIDToTypeID( "T   " );
	        var desc22 = new ActionDescriptor();
	        var id121 = charIDToTypeID( "Hrzn" );
	        var id122 = charIDToTypeID( "#Pxl" );
	        desc22.putUnitDouble( id121, id122, gridx );
	        var id123 = charIDToTypeID( "Vrtc" );
	        var id124 = charIDToTypeID( "#Pxl" );
	        desc22.putUnitDouble( id123, id124, 0 );
	    var id125 = charIDToTypeID( "Pnt " );
	    desc20.putObject( id120, id125, desc22 );
	    var id126 = charIDToTypeID( "Type" );
	    var id127 = charIDToTypeID( "GrdT" );
	    var id128 = charIDToTypeID( "Lnr " );
	    desc20.putEnumerated( id126, id127, id128 );
	    var id129 = charIDToTypeID( "Dthr" );
	    desc20.putBoolean( id129, true );
	    var id130 = charIDToTypeID( "UsMs" );
	    desc20.putBoolean( id130, true );
	    var id131 = charIDToTypeID( "Grad" );
	        var desc23 = new ActionDescriptor();
	        var id132 = charIDToTypeID( "Nm  " );
	        desc23.putString( id132, "Foreground to Background" );
	        var id133 = charIDToTypeID( "GrdF" );
	        var id134 = charIDToTypeID( "GrdF" );
	        var id135 = charIDToTypeID( "CstS" );
	        desc23.putEnumerated( id133, id134, id135 );
	        var id136 = charIDToTypeID( "Intr" );
	        desc23.putDouble( id136, 4096.000000 );
	        var id137 = charIDToTypeID( "Clrs" );
	            var list3 = new ActionList();
	                var desc24 = new ActionDescriptor();
	                var id138 = charIDToTypeID( "Type" );
	                var id139 = charIDToTypeID( "Clry" );
	                var id140 = charIDToTypeID( "FrgC" );
	                desc24.putEnumerated( id138, id139, id140 );
	                var id141 = charIDToTypeID( "Lctn" );
	                desc24.putInteger( id141, 0 );
	                var id142 = charIDToTypeID( "Mdpn" );
	                desc24.putInteger( id142, 50 );
	            var id143 = charIDToTypeID( "Clrt" );
	            list3.putObject( id143, desc24 );
	                var desc25 = new ActionDescriptor();
	                var id144 = charIDToTypeID( "Type" );
	                var id145 = charIDToTypeID( "Clry" );
	                var id146 = charIDToTypeID( "BckC" );
	                desc25.putEnumerated( id144, id145, id146 );
	                var id147 = charIDToTypeID( "Lctn" );
	                desc25.putInteger( id147, 4096 );
	                var id148 = charIDToTypeID( "Mdpn" );
	                desc25.putInteger( id148, 50 );
	            var id149 = charIDToTypeID( "Clrt" );
	            list3.putObject( id149, desc25 );
	        desc23.putList( id137, list3 );
	        var id150 = charIDToTypeID( "Trns" );
	            var list4 = new ActionList();
	                var desc26 = new ActionDescriptor();
	                var id151 = charIDToTypeID( "Opct" );
	                var id152 = charIDToTypeID( "#Prc" );
	                desc26.putUnitDouble( id151, id152, 100.000000 );
	                var id153 = charIDToTypeID( "Lctn" );
	                desc26.putInteger( id153, 0 );
	                var id154 = charIDToTypeID( "Mdpn" );
	                desc26.putInteger( id154, 50 );
	            var id155 = charIDToTypeID( "TrnS" );
	            list4.putObject( id155, desc26 );
	                var desc27 = new ActionDescriptor();
	                var id156 = charIDToTypeID( "Opct" );
	                var id157 = charIDToTypeID( "#Prc" );
	                desc27.putUnitDouble( id156, id157, 100.000000 );
	                var id158 = charIDToTypeID( "Lctn" );
	                desc27.putInteger( id158, 4096 );
	                var id159 = charIDToTypeID( "Mdpn" );
	                desc27.putInteger( id159, 50 );
	            var id160 = charIDToTypeID( "TrnS" );
	            list4.putObject( id160, desc27 );
	        desc23.putList( id150, list4 );
	    var id161 = charIDToTypeID( "Grdn" );
	    desc20.putObject( id131, id161, desc23 );
	executeAction( id113, desc20, DialogModes.NO );	
	
	//===SELECT GREEN CHANNEL===
	var id162 = charIDToTypeID( "slct" );
	    var desc28 = new ActionDescriptor();
	    var id163 = charIDToTypeID( "null" );
	        var ref8 = new ActionReference();
	        var id164 = charIDToTypeID( "Chnl" );
	        var id165 = charIDToTypeID( "Chnl" );
	        var id166 = charIDToTypeID( "Grn " );
	        ref8.putEnumerated( id164, id165, id166 );
	    desc28.putReference( id163, ref8 );
	executeAction( id162, desc28, DialogModes.NO );

	//===GRADIENT FILL FROM (509,511) TO (4,3)===
	var id167 = charIDToTypeID( "Grdn" );
	    var desc29 = new ActionDescriptor();
	    var id168 = charIDToTypeID( "From" );
	        var desc30 = new ActionDescriptor();
	        var id169 = charIDToTypeID( "Hrzn" );
	        var id170 = charIDToTypeID( "#Pxl" );
	        desc30.putUnitDouble( id169, id170, gridx );
	        var id171 = charIDToTypeID( "Vrtc" );
	        var id172 = charIDToTypeID( "#Pxl" );
	        desc30.putUnitDouble( id171, id172, gridy );
	    var id173 = charIDToTypeID( "Pnt " );
	    desc29.putObject( id168, id173, desc30 );
	    var id174 = charIDToTypeID( "T   " );
	        var desc31 = new ActionDescriptor();
	        var id175 = charIDToTypeID( "Hrzn" );
	        var id176 = charIDToTypeID( "#Pxl" );
	        desc31.putUnitDouble( id175, id176, 0 );
	        var id177 = charIDToTypeID( "Vrtc" );
	        var id178 = charIDToTypeID( "#Pxl" );
	        desc31.putUnitDouble( id177, id178, 0 );
	    var id179 = charIDToTypeID( "Pnt " );
	    desc29.putObject( id174, id179, desc31 );
	    var id180 = charIDToTypeID( "Type" );
	    var id181 = charIDToTypeID( "GrdT" );
	    var id182 = charIDToTypeID( "Lnr " );
	    desc29.putEnumerated( id180, id181, id182 );
	    var id183 = charIDToTypeID( "Dthr" );
	    desc29.putBoolean( id183, true );
	    var id184 = charIDToTypeID( "UsMs" );
	    desc29.putBoolean( id184, true );
	    var id185 = charIDToTypeID( "Grad" );
	        var desc32 = new ActionDescriptor();
	        var id186 = charIDToTypeID( "Nm  " );
	        desc32.putString( id186, "Foreground to Background" );
	        var id187 = charIDToTypeID( "GrdF" );
	        var id188 = charIDToTypeID( "GrdF" );
	        var id189 = charIDToTypeID( "CstS" );
	        desc32.putEnumerated( id187, id188, id189 );
	        var id190 = charIDToTypeID( "Intr" );
	        desc32.putDouble( id190, 4096.000000 );
	        var id191 = charIDToTypeID( "Clrs" );
	            var list5 = new ActionList();
	                var desc33 = new ActionDescriptor();
	                var id192 = charIDToTypeID( "Type" );
	                var id193 = charIDToTypeID( "Clry" );
	                var id194 = charIDToTypeID( "FrgC" );
	                desc33.putEnumerated( id192, id193, id194 );
	                var id195 = charIDToTypeID( "Lctn" );
	                desc33.putInteger( id195, 0 );
	                var id196 = charIDToTypeID( "Mdpn" );
	                desc33.putInteger( id196, 50 );
	            var id197 = charIDToTypeID( "Clrt" );
	            list5.putObject( id197, desc33 );
	                var desc34 = new ActionDescriptor();
	                var id198 = charIDToTypeID( "Type" );
	                var id199 = charIDToTypeID( "Clry" );
	                var id200 = charIDToTypeID( "BckC" );
	                desc34.putEnumerated( id198, id199, id200 );
	                var id201 = charIDToTypeID( "Lctn" );
	                desc34.putInteger( id201, 4096 );
	                var id202 = charIDToTypeID( "Mdpn" );
	                desc34.putInteger( id202, 50 );
	            var id203 = charIDToTypeID( "Clrt" );
	            list5.putObject( id203, desc34 );
	        desc32.putList( id191, list5 );
	        var id204 = charIDToTypeID( "Trns" );
	            var list6 = new ActionList();
	                var desc35 = new ActionDescriptor();
	                var id205 = charIDToTypeID( "Opct" );
	                var id206 = charIDToTypeID( "#Prc" );
	                desc35.putUnitDouble( id205, id206, 100.000000 );
	                var id207 = charIDToTypeID( "Lctn" );
	                desc35.putInteger( id207, 0 );
	                var id208 = charIDToTypeID( "Mdpn" );
	                desc35.putInteger( id208, 50 );
	            var id209 = charIDToTypeID( "TrnS" );
	            list6.putObject( id209, desc35 );
	                var desc36 = new ActionDescriptor();
	                var id210 = charIDToTypeID( "Opct" );
	                var id211 = charIDToTypeID( "#Prc" );
	                desc36.putUnitDouble( id210, id211, 100.000000 );
	                var id212 = charIDToTypeID( "Lctn" );
	                desc36.putInteger( id212, 4096 );
	                var id213 = charIDToTypeID( "Mdpn" );
	                desc36.putInteger( id213, 50 );
	            var id214 = charIDToTypeID( "TrnS" );
	            list6.putObject( id214, desc36 );
	        desc32.putList( id204, list6 );
	    var id215 = charIDToTypeID( "Grdn" );
	    desc29.putObject( id185, id215, desc32 );
	executeAction( id167, desc29, DialogModes.NO );

	//===SELECT COMPOSITE (RGB) CHANNEL===
	var id216 = charIDToTypeID( "slct" );
	    var desc37 = new ActionDescriptor();
	    var id217 = charIDToTypeID( "null" );
	        var ref9 = new ActionReference();
	        var id218 = charIDToTypeID( "Chnl" );
	        var id219 = charIDToTypeID( "Chnl" );
	        var id220 = charIDToTypeID( "RGB " );
	        ref9.putEnumerated( id218, id219, id220 );
	    desc37.putReference( id217, ref9 );
	executeAction( id216, desc37, DialogModes.NO );
	}

/*
------------
FUNCTION: pause
------------
*/
function pause(mils) 
	{
	date = new Date();
	var curDate = null;
	do { var curDate = new Date(); } 
	while(curDate-date < mils);
}